# mypy: ignore-errors
# generated by datamodel-codegen:
#   filename:  openai-spec.yaml
#   timestamp: 2024-08-22T02:56:18+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

import pydantic

PYDANTIC_V2 = hasattr(pydantic, "VERSION") and pydantic.VERSION.startswith("2.")
if PYDANTIC_V2:
    from pydantic.v1 import AnyUrl, BaseModel, Extra, Field  # noqa: F401
else:
    from pydantic import AnyUrl, BaseModel, Extra, Field  # type: ignore # noqa: F401
from typing_extensions import Annotated, Literal


class Error(BaseModel):
    code: str
    message: str
    param: str
    type: str


class ErrorResponse(BaseModel):
    error: Error


class DeleteModelResponse(BaseModel):
    id: str
    deleted: bool
    object: str


class Prompt(BaseModel):
    __root__: Annotated[
        Optional[List[int]],
        Field(
            description="The prompt(s) to generate completions for, encoded as a string, array of strings, array of tokens, or array of token arrays.\n\nNote that <|endoftext|> is the document separator that the model sees during training, so if a prompt is not specified the model will generate as if from the beginning of a new document.\n",
            example="[1212, 318, 257, 1332, 13]",
            min_items=1,
        ),
    ] = "<|endoftext|>"


class Prompt1Item(BaseModel):
    __root__: Annotated[List[int], Field(min_items=1)]


class Prompt1(BaseModel):
    __root__: Annotated[
        Optional[List[Prompt1Item]],
        Field(
            description="The prompt(s) to generate completions for, encoded as a string, array of strings, array of tokens, or array of token arrays.\n\nNote that <|endoftext|> is the document separator that the model sees during training, so if a prompt is not specified the model will generate as if from the beginning of a new document.\n",
            example="[[1212, 318, 257, 1332, 13]]",
            min_items=1,
        ),
    ] = "<|endoftext|>"


class Stop(BaseModel):
    __root__: Annotated[
        Optional[List[str]],
        Field(
            description="Up to 4 sequences where the API will stop generating further tokens. The returned text will not contain the stop sequence.\n",
            max_items=4,
            min_items=1,
        ),
    ] = None


class Logprobs(BaseModel):
    text_offset: Optional[List[int]] = None
    token_logprobs: Optional[List[float]] = None
    tokens: Optional[List[str]] = None
    top_logprobs: Optional[List[Dict[str, float]]] = None


class Choice(BaseModel):
    finish_reason: Annotated[
        Literal["stop", "length", "content_filter"],
        Field(
            description="The reason the model stopped generating tokens. This will be `stop` if the model hit a natural stop point or a provided stop sequence,\n`length` if the maximum number of tokens specified in the request was reached,\nor `content_filter` if content was omitted due to a flag from our content filters.\n"
        ),
    ]
    index: int
    logprobs: Logprobs
    text: str


class ChatCompletionRequestMessageContentPartText(BaseModel):
    type: Annotated[Literal["text"], Field(description="The type of the content part.")]
    text: Annotated[str, Field(description="The text content.")]


class ImageUrl(BaseModel):
    url: Annotated[
        AnyUrl,
        Field(description="Either a URL of the image or the base64 encoded image data."),
    ]
    detail: Annotated[
        Optional[Literal["auto", "low", "high"]],
        Field(
            description="Specifies the detail level of the image. Learn more in the [Vision guide](/docs/guides/vision/low-or-high-fidelity-image-understanding)."
        ),
    ] = "auto"


class ChatCompletionRequestMessageContentPartImage(BaseModel):
    type: Annotated[Literal["image_url"], Field(description="The type of the content part.")]
    image_url: ImageUrl


class ChatCompletionRequestMessageContentPartRefusal(BaseModel):
    type: Annotated[Literal["refusal"], Field(description="The type of the content part.")]
    refusal: Annotated[str, Field(description="The refusal message generated by the model.")]


class ChatCompletionRequestSystemMessageContentPart(BaseModel):
    __root__: ChatCompletionRequestMessageContentPartText


class ChatCompletionRequestUserMessageContentPart(BaseModel):
    __root__: Union[
        ChatCompletionRequestMessageContentPartText,
        ChatCompletionRequestMessageContentPartImage,
    ]


class ChatCompletionRequestAssistantMessageContentPart(BaseModel):
    __root__: Union[
        ChatCompletionRequestMessageContentPartText,
        ChatCompletionRequestMessageContentPartRefusal,
    ]


class ChatCompletionRequestToolMessageContentPart(BaseModel):
    __root__: ChatCompletionRequestMessageContentPartText


class Content(BaseModel):
    __root__: Annotated[
        List[ChatCompletionRequestSystemMessageContentPart],
        Field(
            description="An array of content parts with a defined type. For system messages, only type `text` is supported.",
            min_items=1,
            title="Array of content parts",
        ),
    ]


class ChatCompletionRequestSystemMessage(BaseModel):
    content: Annotated[
        Union[str, Content], Field(description="The contents of the system message.")
    ]
    role: Annotated[
        Literal["system"],
        Field(description="The role of the messages author, in this case `system`."),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="An optional name for the participant. Provides the model information to differentiate between participants of the same role."
        ),
    ] = None


class Content1(BaseModel):
    __root__: Annotated[
        List[ChatCompletionRequestUserMessageContentPart],
        Field(
            description="An array of content parts with a defined type, each can be of type `text` or `image_url` when passing in images. You can pass multiple images by adding multiple `image_url` content parts. Image input is only supported when using the `gpt-4o` model.",
            min_items=1,
            title="Array of content parts",
        ),
    ]


class ChatCompletionRequestUserMessage(BaseModel):
    content: Annotated[
        Union[str, Content1], Field(description="The contents of the user message.\n")
    ]
    role: Annotated[
        Literal["user"],
        Field(description="The role of the messages author, in this case `user`."),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="An optional name for the participant. Provides the model information to differentiate between participants of the same role."
        ),
    ] = None


class Content2(BaseModel):
    __root__: Annotated[
        Optional[List[ChatCompletionRequestAssistantMessageContentPart]],
        Field(
            description="An array of content parts with a defined type. Can be one or more of type `text`, or exactly one of type `refusal`.",
            min_items=1,
            title="Array of content parts",
        ),
    ] = None


class FunctionCall(BaseModel):
    arguments: Annotated[
        str,
        Field(
            description="The arguments to call the function with, as generated by the model in JSON format. Note that the model does not always generate valid JSON, and may hallucinate parameters not defined by your function schema. Validate the arguments in your code before calling your function."
        ),
    ]
    name: Annotated[str, Field(description="The name of the function to call.")]


class Content3(BaseModel):
    __root__: Annotated[
        List[ChatCompletionRequestToolMessageContentPart],
        Field(
            description="An array of content parts with a defined type. For tool messages, only type `text` is supported.",
            min_items=1,
            title="Array of content parts",
        ),
    ]


class ChatCompletionRequestToolMessage(BaseModel):
    role: Annotated[
        Literal["tool"],
        Field(description="The role of the messages author, in this case `tool`."),
    ]
    content: Annotated[Union[str, Content3], Field(description="The contents of the tool message.")]
    tool_call_id: Annotated[str, Field(description="Tool call that this message is responding to.")]


class ChatCompletionRequestFunctionMessage(BaseModel):
    role: Annotated[
        Literal["function"],
        Field(description="The role of the messages author, in this case `function`."),
    ]
    content: Annotated[str, Field(description="The contents of the function message.")]
    name: Annotated[str, Field(description="The name of the function to call.")]


class FunctionParameters(BaseModel):
    pass

    class Config:
        extra = Extra.allow


class ChatCompletionFunctions(BaseModel):
    description: Annotated[
        Optional[str],
        Field(
            description="A description of what the function does, used by the model to choose when and how to call the function."
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."
        ),
    ]
    parameters: Optional[FunctionParameters] = None


class ChatCompletionFunctionCallOption(BaseModel):
    name: Annotated[str, Field(description="The name of the function to call.")]


class FunctionObject(BaseModel):
    description: Annotated[
        Optional[str],
        Field(
            description="A description of what the function does, used by the model to choose when and how to call the function."
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."
        ),
    ]
    parameters: Optional[FunctionParameters] = None
    strict: Annotated[
        Optional[bool],
        Field(
            description="Whether to enable strict schema adherence when generating the function call. If set to true, the model will follow the exact schema defined in the `parameters` field. Only a subset of JSON Schema is supported when `strict` is `true`. Learn more about Structured Outputs in the [function calling guide](docs/guides/function-calling)."
        ),
    ] = False


class ResponseFormatText(BaseModel):
    type: Annotated[
        Literal["text"],
        Field(description="The type of response format being defined: `text`"),
    ]


class ResponseFormatJsonObject(BaseModel):
    type: Annotated[
        Literal["json_object"],
        Field(description="The type of response format being defined: `json_object`"),
    ]


class ResponseFormatJsonSchemaSchema(BaseModel):
    pass

    class Config:
        extra = Extra.allow


class JsonSchema(BaseModel):
    description: Annotated[
        Optional[str],
        Field(
            description="A description of what the response format is for, used by the model to determine how to respond in the format."
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="The name of the response format. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."
        ),
    ]
    schema_: Annotated[Optional[ResponseFormatJsonSchemaSchema], Field(alias="schema")] = None
    strict: Annotated[
        Optional[bool],
        Field(
            description="Whether to enable strict schema adherence when generating the output. If set to true, the model will always follow the exact schema defined in the `schema` field. Only a subset of JSON Schema is supported when `strict` is `true`. To learn more, read the [Structured Outputs guide](/docs/guides/structured-outputs)."
        ),
    ] = False


class ResponseFormatJsonSchema(BaseModel):
    type: Annotated[
        Literal["json_schema"],
        Field(description="The type of response format being defined: `json_schema`"),
    ]
    json_schema: JsonSchema


class Function(BaseModel):
    name: Annotated[str, Field(description="The name of the function to call.")]


class ChatCompletionNamedToolChoice(BaseModel):
    type: Annotated[
        Literal["function"],
        Field(description="The type of the tool. Currently, only `function` is supported."),
    ]
    function: Function


class ParallelToolCalls(BaseModel):
    __root__: Annotated[
        bool,
        Field(
            description="Whether to enable [parallel function calling](/docs/guides/function-calling/parallel-function-calling) during tool use."
        ),
    ]


class Function1(BaseModel):
    name: Annotated[str, Field(description="The name of the function to call.")]
    arguments: Annotated[
        str,
        Field(
            description="The arguments to call the function with, as generated by the model in JSON format. Note that the model does not always generate valid JSON, and may hallucinate parameters not defined by your function schema. Validate the arguments in your code before calling your function."
        ),
    ]


class ChatCompletionMessageToolCall(BaseModel):
    id: Annotated[str, Field(description="The ID of the tool call.")]
    type: Annotated[
        Literal["function"],
        Field(description="The type of the tool. Currently, only `function` is supported."),
    ]
    function: Annotated[Function1, Field(description="The function that the model called.")]


class Function2(BaseModel):
    name: Annotated[Optional[str], Field(description="The name of the function to call.")] = None
    arguments: Annotated[
        Optional[str],
        Field(
            description="The arguments to call the function with, as generated by the model in JSON format. Note that the model does not always generate valid JSON, and may hallucinate parameters not defined by your function schema. Validate the arguments in your code before calling your function."
        ),
    ] = None


class ChatCompletionMessageToolCallChunk(BaseModel):
    index: int
    id: Annotated[Optional[str], Field(description="The ID of the tool call.")] = None
    type: Annotated[
        Optional[Literal["function"]],
        Field(description="The type of the tool. Currently, only `function` is supported."),
    ] = None
    function: Optional[Function2] = None


class ChatCompletionRole(BaseModel):
    __root__: Annotated[
        Literal["system", "user", "assistant", "tool", "function"],
        Field(description="The role of the author of a message"),
    ]


class ChatCompletionStreamOptions(BaseModel):
    include_usage: Annotated[
        Optional[bool],
        Field(
            description="If set, an additional chunk will be streamed before the `data: [DONE]` message. The `usage` field on this chunk shows the token usage statistics for the entire request, and the `choices` field will always be an empty array. All other chunks will also include a `usage` field, but with a null value.\n"
        ),
    ] = None


class FunctionCall2(BaseModel):
    arguments: Annotated[
        Optional[str],
        Field(
            description="The arguments to call the function with, as generated by the model in JSON format. Note that the model does not always generate valid JSON, and may hallucinate parameters not defined by your function schema. Validate the arguments in your code before calling your function."
        ),
    ] = None
    name: Annotated[Optional[str], Field(description="The name of the function to call.")] = None


class ChatCompletionStreamResponseDelta(BaseModel):
    content: Annotated[
        Optional[str], Field(description="The contents of the chunk message.")
    ] = None
    function_call: Annotated[
        Optional[FunctionCall2],
        Field(
            description="Deprecated and replaced by `tool_calls`. The name and arguments of a function that should be called, as generated by the model."
        ),
    ] = None
    tool_calls: Optional[List[ChatCompletionMessageToolCallChunk]] = None
    role: Annotated[
        Optional[Literal["system", "user", "assistant", "tool"]],
        Field(description="The role of the author of this message."),
    ] = None
    refusal: Annotated[
        Optional[str], Field(description="The refusal message generated by the model.")
    ] = None


class Stop1(BaseModel):
    __root__: Annotated[
        List[str],
        Field(
            description="Up to 4 sequences where the API will stop generating further tokens.\n",
            max_items=4,
            min_items=1,
        ),
    ]


class TopLogprob(BaseModel):
    token: Annotated[str, Field(description="The token.")]
    logprob: Annotated[
        float,
        Field(
            description="The log probability of this token, if it is within the top 20 most likely tokens. Otherwise, the value `-9999.0` is used to signify that the token is very unlikely."
        ),
    ]
    bytes: Annotated[
        List[int],
        Field(
            description="A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."
        ),
    ]


class ChatCompletionTokenLogprob(BaseModel):
    token: Annotated[str, Field(description="The token.")]
    logprob: Annotated[
        float,
        Field(
            description="The log probability of this token, if it is within the top 20 most likely tokens. Otherwise, the value `-9999.0` is used to signify that the token is very unlikely."
        ),
    ]
    bytes: Annotated[
        List[int],
        Field(
            description="A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."
        ),
    ]
    top_logprobs: Annotated[
        List[TopLogprob],
        Field(
            description="List of the most likely tokens and their log probability, at this token position. In rare cases, there may be fewer than the number of requested `top_logprobs` returned."
        ),
    ]


class Logprobs2(BaseModel):
    content: Annotated[
        List[ChatCompletionTokenLogprob],
        Field(description="A list of message content tokens with log probability information."),
    ]
    refusal: Annotated[
        List[ChatCompletionTokenLogprob],
        Field(description="A list of message refusal tokens with log probability information."),
    ]


class Choice3(BaseModel):
    delta: ChatCompletionStreamResponseDelta
    logprobs: Annotated[
        Optional[Logprobs2],
        Field(description="Log probability information for the choice."),
    ] = None
    finish_reason: Annotated[
        Literal["stop", "length", "tool_calls", "content_filter", "function_call"],
        Field(
            description="The reason the model stopped generating tokens. This will be `stop` if the model hit a natural stop point or a provided stop sequence,\n`length` if the maximum number of tokens specified in the request was reached,\n`content_filter` if content was omitted due to a flag from our content filters,\n`tool_calls` if the model called a tool, or `function_call` (deprecated) if the model called a function.\n"
        ),
    ]
    index: Annotated[int, Field(description="The index of the choice in the list of choices.")]


class Usage(BaseModel):
    completion_tokens: Annotated[
        int, Field(description="Number of tokens in the generated completion.")
    ]
    prompt_tokens: Annotated[int, Field(description="Number of tokens in the prompt.")]
    total_tokens: Annotated[
        int,
        Field(description="Total number of tokens used in the request (prompt + completion)."),
    ]


class CreateChatCompletionStreamResponse(BaseModel):
    id: Annotated[
        str,
        Field(
            description="A unique identifier for the chat completion. Each chunk has the same ID."
        ),
    ]
    choices: Annotated[
        List[Choice3],
        Field(
            description='A list of chat completion choices. Can contain more than one elements if `n` is greater than 1. Can also be empty for the\nlast chunk if you set `stream_options: {"include_usage": true}`.\n'
        ),
    ]
    created: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) of when the chat completion was created. Each chunk has the same timestamp."
        ),
    ]
    model: Annotated[str, Field(description="The model to generate the completion.")]
    service_tier: Annotated[
        Optional[Literal["scale", "default"]],
        Field(
            description="The service tier used for processing the request. This field is only included if the `service_tier` parameter is specified in the request.",
            example="scale",
        ),
    ] = None
    system_fingerprint: Annotated[
        Optional[str],
        Field(
            description="This fingerprint represents the backend configuration that the model runs with.\nCan be used in conjunction with the `seed` request parameter to understand when backend changes have been made that might impact determinism.\n"
        ),
    ] = None
    object: Annotated[
        Literal["chat.completion.chunk"],
        Field(description="The object type, which is always `chat.completion.chunk`."),
    ]
    usage: Annotated[
        Optional[Usage],
        Field(
            description='An optional field that will only be present when you set `stream_options: {"include_usage": true}` in your request.\nWhen present, it contains a null value except for the last chunk which contains the token usage statistics for the entire request.\n'
        ),
    ] = None


class CreateChatCompletionImageResponse(BaseModel):
    pass


class CreateImageRequest(BaseModel):
    prompt: Annotated[
        str,
        Field(
            description="A text description of the desired image(s). The maximum length is 1000 characters for `dall-e-2` and 4000 characters for `dall-e-3`.",
            example="A cute baby sea otter",
        ),
    ]
    model: Annotated[
        Optional[Union[str, Literal["dall-e-2", "dall-e-3"]]],
        Field(description="The model to use for image generation.", example="dall-e-3"),
    ] = "dall-e-2"
    n: Annotated[
        Optional[int],
        Field(
            description="The number of images to generate. Must be between 1 and 10. For `dall-e-3`, only `n=1` is supported.",
            example=1,
            ge=1,
            le=10,
        ),
    ] = 1
    quality: Annotated[
        Optional[Literal["standard", "hd"]],
        Field(
            description="The quality of the image that will be generated. `hd` creates images with finer details and greater consistency across the image. This param is only supported for `dall-e-3`.",
            example="standard",
        ),
    ] = "standard"
    response_format: Annotated[
        Optional[Literal["url", "b64_json"]],
        Field(
            description="The format in which the generated images are returned. Must be one of `url` or `b64_json`. URLs are only valid for 60 minutes after the image has been generated.",
            example="url",
        ),
    ] = "url"
    size: Annotated[
        Optional[Literal["256x256", "512x512", "1024x1024", "1792x1024", "1024x1792"]],
        Field(
            description="The size of the generated images. Must be one of `256x256`, `512x512`, or `1024x1024` for `dall-e-2`. Must be one of `1024x1024`, `1792x1024`, or `1024x1792` for `dall-e-3` models.",
            example="1024x1024",
        ),
    ] = "1024x1024"
    style: Annotated[
        Optional[Literal["vivid", "natural"]],
        Field(
            description="The style of the generated images. Must be one of `vivid` or `natural`. Vivid causes the model to lean towards generating hyper-real and dramatic images. Natural causes the model to produce more natural, less hyper-real looking images. This param is only supported for `dall-e-3`.",
            example="vivid",
        ),
    ] = "vivid"
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None


class Image(BaseModel):
    b64_json: Annotated[
        Optional[str],
        Field(
            description="The base64-encoded JSON of the generated image, if `response_format` is `b64_json`."
        ),
    ] = None
    url: Annotated[
        Optional[str],
        Field(
            description="The URL of the generated image, if `response_format` is `url` (default)."
        ),
    ] = None
    revised_prompt: Annotated[
        Optional[str],
        Field(
            description="The prompt that was used to generate the image, if there was any revision to the prompt."
        ),
    ] = None


class CreateImageEditRequest(BaseModel):
    image: Annotated[
        bytes,
        Field(
            description="The image to edit. Must be a valid PNG file, less than 4MB, and square. If mask is not provided, image must have transparency, which will be used as the mask."
        ),
    ]
    prompt: Annotated[
        str,
        Field(
            description="A text description of the desired image(s). The maximum length is 1000 characters.",
            example="A cute baby sea otter wearing a beret",
        ),
    ]
    mask: Annotated[
        Optional[bytes],
        Field(
            description="An additional image whose fully transparent areas (e.g. where alpha is zero) indicate where `image` should be edited. Must be a valid PNG file, less than 4MB, and have the same dimensions as `image`."
        ),
    ] = None
    model: Annotated[
        Optional[Union[str, Literal["dall-e-2"]]],
        Field(
            description="The model to use for image generation. Only `dall-e-2` is supported at this time.",
            example="dall-e-2",
        ),
    ] = "dall-e-2"
    n: Annotated[
        Optional[int],
        Field(
            description="The number of images to generate. Must be between 1 and 10.",
            example=1,
            ge=1,
            le=10,
        ),
    ] = 1
    size: Annotated[
        Optional[Literal["256x256", "512x512", "1024x1024"]],
        Field(
            description="The size of the generated images. Must be one of `256x256`, `512x512`, or `1024x1024`.",
            example="1024x1024",
        ),
    ] = "1024x1024"
    response_format: Annotated[
        Optional[Literal["url", "b64_json"]],
        Field(
            description="The format in which the generated images are returned. Must be one of `url` or `b64_json`. URLs are only valid for 60 minutes after the image has been generated.",
            example="url",
        ),
    ] = "url"
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None


class CreateImageVariationRequest(BaseModel):
    image: Annotated[
        bytes,
        Field(
            description="The image to use as the basis for the variation(s). Must be a valid PNG file, less than 4MB, and square."
        ),
    ]
    model: Annotated[
        Optional[Union[str, Literal["dall-e-2"]]],
        Field(
            description="The model to use for image generation. Only `dall-e-2` is supported at this time.",
            example="dall-e-2",
        ),
    ] = "dall-e-2"
    n: Annotated[
        Optional[int],
        Field(
            description="The number of images to generate. Must be between 1 and 10. For `dall-e-3`, only `n=1` is supported.",
            example=1,
            ge=1,
            le=10,
        ),
    ] = 1
    response_format: Annotated[
        Optional[Literal["url", "b64_json"]],
        Field(
            description="The format in which the generated images are returned. Must be one of `url` or `b64_json`. URLs are only valid for 60 minutes after the image has been generated.",
            example="url",
        ),
    ] = "url"
    size: Annotated[
        Optional[Literal["256x256", "512x512", "1024x1024"]],
        Field(
            description="The size of the generated images. Must be one of `256x256`, `512x512`, or `1024x1024`.",
            example="1024x1024",
        ),
    ] = "1024x1024"
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None


class CreateModerationRequest(BaseModel):
    input: Annotated[Union[str, List[str]], Field(description="The input text to classify")]
    model: Annotated[
        Optional[Union[str, Literal["text-moderation-latest", "text-moderation-stable"]]],
        Field(
            description="Two content moderations models are available: `text-moderation-stable` and `text-moderation-latest`.\n\nThe default is `text-moderation-latest` which will be automatically upgraded over time. This ensures you are always using our most accurate model. If you use `text-moderation-stable`, we will provide advanced notice before updating the model. Accuracy of `text-moderation-stable` may be slightly lower than for `text-moderation-latest`.\n",
            example="text-moderation-stable",
        ),
    ] = "text-moderation-latest"


class Categories(BaseModel):
    hate: Annotated[
        bool,
        Field(
            description="Content that expresses, incites, or promotes hate based on race, gender, ethnicity, religion, nationality, sexual orientation, disability status, or caste. Hateful content aimed at non-protected groups (e.g., chess players) is harassment."
        ),
    ]
    hate_threatening: Annotated[
        bool,
        Field(
            alias="hate/threatening",
            description="Hateful content that also includes violence or serious harm towards the targeted group based on race, gender, ethnicity, religion, nationality, sexual orientation, disability status, or caste.",
        ),
    ]
    harassment: Annotated[
        bool,
        Field(
            description="Content that expresses, incites, or promotes harassing language towards any target."
        ),
    ]
    harassment_threatening: Annotated[
        bool,
        Field(
            alias="harassment/threatening",
            description="Harassment content that also includes violence or serious harm towards any target.",
        ),
    ]
    self_harm: Annotated[
        bool,
        Field(
            alias="self-harm",
            description="Content that promotes, encourages, or depicts acts of self-harm, such as suicide, cutting, and eating disorders.",
        ),
    ]
    self_harm_intent: Annotated[
        bool,
        Field(
            alias="self-harm/intent",
            description="Content where the speaker expresses that they are engaging or intend to engage in acts of self-harm, such as suicide, cutting, and eating disorders.",
        ),
    ]
    self_harm_instructions: Annotated[
        bool,
        Field(
            alias="self-harm/instructions",
            description="Content that encourages performing acts of self-harm, such as suicide, cutting, and eating disorders, or that gives instructions or advice on how to commit such acts.",
        ),
    ]
    sexual: Annotated[
        bool,
        Field(
            description="Content meant to arouse sexual excitement, such as the description of sexual activity, or that promotes sexual services (excluding sex education and wellness)."
        ),
    ]
    sexual_minors: Annotated[
        bool,
        Field(
            alias="sexual/minors",
            description="Sexual content that includes an individual who is under 18 years old.",
        ),
    ]
    violence: Annotated[
        bool,
        Field(description="Content that depicts death, violence, or physical injury."),
    ]
    violence_graphic: Annotated[
        bool,
        Field(
            alias="violence/graphic",
            description="Content that depicts death, violence, or physical injury in graphic detail.",
        ),
    ]


class CategoryScores(BaseModel):
    hate: Annotated[float, Field(description="The score for the category 'hate'.")]
    hate_threatening: Annotated[
        float,
        Field(
            alias="hate/threatening",
            description="The score for the category 'hate/threatening'.",
        ),
    ]
    harassment: Annotated[float, Field(description="The score for the category 'harassment'.")]
    harassment_threatening: Annotated[
        float,
        Field(
            alias="harassment/threatening",
            description="The score for the category 'harassment/threatening'.",
        ),
    ]
    self_harm: Annotated[
        float,
        Field(alias="self-harm", description="The score for the category 'self-harm'."),
    ]
    self_harm_intent: Annotated[
        float,
        Field(
            alias="self-harm/intent",
            description="The score for the category 'self-harm/intent'.",
        ),
    ]
    self_harm_instructions: Annotated[
        float,
        Field(
            alias="self-harm/instructions",
            description="The score for the category 'self-harm/instructions'.",
        ),
    ]
    sexual: Annotated[float, Field(description="The score for the category 'sexual'.")]
    sexual_minors: Annotated[
        float,
        Field(
            alias="sexual/minors",
            description="The score for the category 'sexual/minors'.",
        ),
    ]
    violence: Annotated[float, Field(description="The score for the category 'violence'.")]
    violence_graphic: Annotated[
        float,
        Field(
            alias="violence/graphic",
            description="The score for the category 'violence/graphic'.",
        ),
    ]


class Result(BaseModel):
    flagged: Annotated[bool, Field(description="Whether any of the below categories are flagged.")]
    categories: Annotated[
        Categories,
        Field(description="A list of the categories, and whether they are flagged or not."),
    ]
    category_scores: Annotated[
        CategoryScores,
        Field(
            description="A list of the categories along with their scores as predicted by model."
        ),
    ]


class CreateModerationResponse(BaseModel):
    id: Annotated[str, Field(description="The unique identifier for the moderation request.")]
    model: Annotated[str, Field(description="The model used to generate the moderation results.")]
    results: Annotated[List[Result], Field(description="A list of moderation objects.")]


class CreateFileRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file: Annotated[bytes, Field(description="The File object (not file name) to be uploaded.\n")]
    purpose: Annotated[
        Literal["assistants", "batch", "fine-tune", "vision"],
        Field(
            description='The intended purpose of the uploaded file.\n\nUse "assistants" for [Assistants](/docs/api-reference/assistants) and [Message](/docs/api-reference/messages) files, "vision" for Assistants image file inputs, "batch" for [Batch API](/docs/guides/batch), and "fine-tune" for [Fine-tuning](/docs/api-reference/fine-tuning).\n'
        ),
    ]


class DeleteFileResponse(BaseModel):
    id: str
    object: Literal["file"]
    deleted: bool


class CreateUploadRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    filename: Annotated[str, Field(description="The name of the file to upload.\n")]
    purpose: Annotated[
        Literal["assistants", "batch", "fine-tune", "vision"],
        Field(
            description="The intended purpose of the uploaded file.\n\nSee the [documentation on File purposes](/docs/api-reference/files/create#files-create-purpose).\n"
        ),
    ]
    bytes: Annotated[int, Field(description="The number of bytes in the file you are uploading.\n")]
    mime_type: Annotated[
        str,
        Field(
            description="The MIME type of the file.\n\nThis must fall within the supported MIME types for your file purpose. See the supported MIME types for assistants and vision.\n"
        ),
    ]


class AddUploadPartRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    data: Annotated[bytes, Field(description="The chunk of bytes for this Part.\n")]


class CompleteUploadRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    part_ids: Annotated[List[str], Field(description="The ordered list of Part IDs.\n")]
    md5: Annotated[
        Optional[str],
        Field(
            description="The optional md5 checksum for the file contents to verify if the bytes uploaded matches what you expect.\n"
        ),
    ] = None


class CancelUploadRequest(BaseModel):
    pass

    class Config:
        extra = Extra.forbid


class BatchSize(BaseModel):
    __root__: Annotated[
        int,
        Field(
            description="Number of examples in each batch. A larger batch size means that model parameters\nare updated less frequently, but with lower variance.\n",
            ge=1,
            le=256,
        ),
    ]


class LearningRateMultiplier(BaseModel):
    __root__: Annotated[
        float,
        Field(
            description="Scaling factor for the learning rate. A smaller learning rate may be useful to avoid\noverfitting.\n",
            gt=0.0,
        ),
    ]


class NEpochs(BaseModel):
    __root__: Annotated[
        int,
        Field(
            description="The number of epochs to train the model for. An epoch refers to one full cycle\nthrough the training dataset.\n",
            ge=1,
            le=50,
        ),
    ]


class Hyperparameters(BaseModel):
    batch_size: Annotated[
        Optional[Union[Literal["auto"], BatchSize]],
        Field(
            description="Number of examples in each batch. A larger batch size means that model parameters\nare updated less frequently, but with lower variance.\n"
        ),
    ] = "auto"
    learning_rate_multiplier: Annotated[
        Optional[Union[Literal["auto"], LearningRateMultiplier]],
        Field(
            description="Scaling factor for the learning rate. A smaller learning rate may be useful to avoid\noverfitting.\n"
        ),
    ] = "auto"
    n_epochs: Annotated[
        Optional[Union[Literal["auto"], NEpochs]],
        Field(
            description="The number of epochs to train the model for. An epoch refers to one full cycle\nthrough the training dataset.\n"
        ),
    ] = "auto"


class Wandb(BaseModel):
    project: Annotated[
        str,
        Field(
            description="The name of the project that the new run will be created under.\n",
            example="my-wandb-project",
        ),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="A display name to set for the run. If not set, we will use the Job ID as the name.\n"
        ),
    ] = None
    entity: Annotated[
        Optional[str],
        Field(
            description="The entity to use for the run. This allows you to set the team or username of the WandB user that you would\nlike associated with the run. If not set, the default entity for the registered WandB API key is used.\n"
        ),
    ] = None
    tags: Annotated[
        Optional[List[str]],
        Field(
            description='A list of tags to be attached to the newly created run. These tags are passed through directly to WandB. Some\ndefault tags are generated by OpenAI: "openai/finetune", "openai/{base-model}", "openai/{ftjob-abcdef}".\n'
        ),
    ] = None


class Integration(BaseModel):
    type: Annotated[
        Literal["wandb"],
        Field(
            description='The type of integration to enable. Currently, only "wandb" (Weights and Biases) is supported.\n'
        ),
    ]
    wandb: Annotated[
        Wandb,
        Field(
            description="The settings for your integration with Weights and Biases. This payload specifies the project that\nmetrics will be sent to. Optionally, you can set an explicit display name for your run, add tags\nto your run, and set a default entity (team, username, etc) to be associated with your run.\n"
        ),
    ]


class CreateFineTuningJobRequest(BaseModel):
    model: Annotated[
        Union[str, Literal["babbage-002", "davinci-002", "gpt-3.5-turbo", "gpt-4o-mini"]],
        Field(
            description="The name of the model to fine-tune. You can select one of the\n[supported models](/docs/guides/fine-tuning/which-models-can-be-fine-tuned).\n",
            example="gpt-4o-mini",
        ),
    ]
    training_file: Annotated[
        str,
        Field(
            description="The ID of an uploaded file that contains training data.\n\nSee [upload file](/docs/api-reference/files/create) for how to upload a file.\n\nYour dataset must be formatted as a JSONL file. Additionally, you must upload your file with the purpose `fine-tune`.\n\nThe contents of the file should differ depending on if the model uses the [chat](/docs/api-reference/fine-tuning/chat-input) or [completions](/docs/api-reference/fine-tuning/completions-input) format.\n\nSee the [fine-tuning guide](/docs/guides/fine-tuning) for more details.\n",
            example="file-abc123",
        ),
    ]
    hyperparameters: Annotated[
        Optional[Hyperparameters],
        Field(description="The hyperparameters used for the fine-tuning job."),
    ] = None
    suffix: Annotated[
        Optional[str],
        Field(
            description='A string of up to 18 characters that will be added to your fine-tuned model name.\n\nFor example, a `suffix` of "custom-model-name" would produce a model name like `ft:gpt-4o-mini:openai:custom-model-name:7p4lURel`.\n',
            max_length=40,
            min_length=1,
        ),
    ] = None
    validation_file: Annotated[
        Optional[str],
        Field(
            description="The ID of an uploaded file that contains validation data.\n\nIf you provide this file, the data is used to generate validation\nmetrics periodically during fine-tuning. These metrics can be viewed in\nthe fine-tuning results file.\nThe same data should not be present in both train and validation files.\n\nYour dataset must be formatted as a JSONL file. You must upload your file with the purpose `fine-tune`.\n\nSee the [fine-tuning guide](/docs/guides/fine-tuning) for more details.\n",
            example="file-abc123",
        ),
    ] = None
    integrations: Annotated[
        Optional[List[Integration]],
        Field(description="A list of integrations to enable for your fine-tuning job."),
    ] = None
    seed: Annotated[
        Optional[int],
        Field(
            description="The seed controls the reproducibility of the job. Passing in the same seed and job parameters should produce the same results, but may differ in rare cases.\nIf a seed is not specified, one will be generated for you.\n",
            example=42,
            ge=0,
            le=2147483647,
        ),
    ] = None


class Input(BaseModel):
    __root__: Annotated[
        List[str],
        Field(
            description="The array of strings that will be turned into an embedding.",
            example="The quick brown fox jumped over the lazy dog",
            max_items=2048,
            min_items=1,
            title="array",
        ),
    ]


class Input1(BaseModel):
    __root__: Annotated[
        List[int],
        Field(
            description="The array of integers that will be turned into an embedding.",
            example="[1212, 318, 257, 1332, 13]",
            max_items=2048,
            min_items=1,
            title="array",
        ),
    ]


class Input2Item(BaseModel):
    __root__: Annotated[List[int], Field(min_items=1)]


class Input2(BaseModel):
    __root__: Annotated[
        List[Input2Item],
        Field(
            description="The array of arrays containing integers that will be turned into an embedding.",
            example="[[1212, 318, 257, 1332, 13]]",
            max_items=2048,
            min_items=1,
            title="array",
        ),
    ]


class CreateEmbeddingRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    input: Annotated[
        Union[str, Input, Input1, Input2],
        Field(
            description="Input text to embed, encoded as a string or array of tokens. To embed multiple inputs in a single request, pass an array of strings or array of token arrays. The input must not exceed the max input tokens for the model (8192 tokens for `text-embedding-ada-002`), cannot be an empty string, and any array must be 2048 dimensions or less. [Example Python code](https://cookbook.openai.com/examples/how_to_count_tokens_with_tiktoken) for counting tokens.\n",
            example="The quick brown fox jumped over the lazy dog",
        ),
    ]
    model: Annotated[
        Union[
            str,
            Literal[
                "text-embedding-ada-002",
                "text-embedding-3-small",
                "text-embedding-3-large",
            ],
        ],
        Field(
            description="ID of the model to use. You can use the [List models](/docs/api-reference/models/list) API to see all of your available models, or see our [Model overview](/docs/models/overview) for descriptions of them.\n",
            example="text-embedding-3-small",
        ),
    ]
    encoding_format: Annotated[
        Optional[Literal["float", "base64"]],
        Field(
            description="The format to return the embeddings in. Can be either `float` or [`base64`](https://pypi.org/project/pybase64/).",
            example="float",
        ),
    ] = "float"
    dimensions: Annotated[
        Optional[int],
        Field(
            description="The number of dimensions the resulting output embeddings should have. Only supported in `text-embedding-3` and later models.\n",
            ge=1,
        ),
    ] = None
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None


class Usage1(BaseModel):
    prompt_tokens: Annotated[int, Field(description="The number of tokens used by the prompt.")]
    total_tokens: Annotated[
        int, Field(description="The total number of tokens used by the request.")
    ]


class CreateTranscriptionRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file: Annotated[
        bytes,
        Field(
            description="The audio file object (not file name) to transcribe, in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm.\n"
        ),
    ]
    model: Annotated[
        Union[str, Literal["whisper-1"]],
        Field(
            description="ID of the model to use. Only `whisper-1` (which is powered by our open source Whisper V2 model) is currently available.\n",
            example="whisper-1",
        ),
    ]
    language: Annotated[
        Optional[str],
        Field(
            description="The language of the input audio. Supplying the input language in [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) format will improve accuracy and latency.\n"
        ),
    ] = None
    prompt: Annotated[
        Optional[str],
        Field(
            description="An optional text to guide the model's style or continue a previous audio segment. The [prompt](/docs/guides/speech-to-text/prompting) should match the audio language.\n"
        ),
    ] = None
    response_format: Annotated[
        Optional[Literal["json", "text", "srt", "verbose_json", "vtt"]],
        Field(
            description="The format of the transcript output, in one of these options: `json`, `text`, `srt`, `verbose_json`, or `vtt`.\n"
        ),
    ] = "json"
    temperature: Annotated[
        Optional[float],
        Field(
            description="The sampling temperature, between 0 and 1. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. If set to 0, the model will use [log probability](https://en.wikipedia.org/wiki/Log_probability) to automatically increase the temperature until certain thresholds are hit.\n"
        ),
    ] = 0
    timestamp_granularities__: Annotated[
        Optional[List[Literal["word", "segment"]]],
        Field(
            alias="timestamp_granularities[]",
            description="The timestamp granularities to populate for this transcription. `response_format` must be set `verbose_json` to use timestamp granularities. Either or both of these options are supported: `word`, or `segment`. Note: There is no additional latency for segment timestamps, but generating word timestamps incurs additional latency.\n",
        ),
    ] = ["segment"]


class CreateTranscriptionResponseJson(BaseModel):
    text: Annotated[str, Field(description="The transcribed text.")]


class TranscriptionSegment(BaseModel):
    id: Annotated[int, Field(description="Unique identifier of the segment.")]
    seek: Annotated[int, Field(description="Seek offset of the segment.")]
    start: Annotated[float, Field(description="Start time of the segment in seconds.")]
    end: Annotated[float, Field(description="End time of the segment in seconds.")]
    text: Annotated[str, Field(description="Text content of the segment.")]
    tokens: Annotated[List[int], Field(description="Array of token IDs for the text content.")]
    temperature: Annotated[
        float,
        Field(description="Temperature parameter used for generating the segment."),
    ]
    avg_logprob: Annotated[
        float,
        Field(
            description="Average logprob of the segment. If the value is lower than -1, consider the logprobs failed."
        ),
    ]
    compression_ratio: Annotated[
        float,
        Field(
            description="Compression ratio of the segment. If the value is greater than 2.4, consider the compression failed."
        ),
    ]
    no_speech_prob: Annotated[
        float,
        Field(
            description="Probability of no speech in the segment. If the value is higher than 1.0 and the `avg_logprob` is below -1, consider this segment silent."
        ),
    ]


class TranscriptionWord(BaseModel):
    word: Annotated[str, Field(description="The text content of the word.")]
    start: Annotated[float, Field(description="Start time of the word in seconds.")]
    end: Annotated[float, Field(description="End time of the word in seconds.")]


class CreateTranscriptionResponseVerboseJson(BaseModel):
    language: Annotated[str, Field(description="The language of the input audio.")]
    duration: Annotated[str, Field(description="The duration of the input audio.")]
    text: Annotated[str, Field(description="The transcribed text.")]
    words: Annotated[
        Optional[List[TranscriptionWord]],
        Field(description="Extracted words and their corresponding timestamps."),
    ] = None
    segments: Annotated[
        Optional[List[TranscriptionSegment]],
        Field(description="Segments of the transcribed text and their corresponding details."),
    ] = None


class CreateTranslationRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file: Annotated[
        bytes,
        Field(
            description="The audio file object (not file name) translate, in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm.\n"
        ),
    ]
    model: Annotated[
        Union[str, Literal["whisper-1"]],
        Field(
            description="ID of the model to use. Only `whisper-1` (which is powered by our open source Whisper V2 model) is currently available.\n",
            example="whisper-1",
        ),
    ]
    prompt: Annotated[
        Optional[str],
        Field(
            description="An optional text to guide the model's style or continue a previous audio segment. The [prompt](/docs/guides/speech-to-text/prompting) should be in English.\n"
        ),
    ] = None
    response_format: Annotated[
        Optional[str],
        Field(
            description="The format of the transcript output, in one of these options: `json`, `text`, `srt`, `verbose_json`, or `vtt`.\n"
        ),
    ] = "json"
    temperature: Annotated[
        Optional[float],
        Field(
            description="The sampling temperature, between 0 and 1. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. If set to 0, the model will use [log probability](https://en.wikipedia.org/wiki/Log_probability) to automatically increase the temperature until certain thresholds are hit.\n"
        ),
    ] = 0


class CreateTranslationResponseJson(BaseModel):
    text: str


class CreateTranslationResponseVerboseJson(BaseModel):
    language: Annotated[
        str,
        Field(description="The language of the output translation (always `english`)."),
    ]
    duration: Annotated[str, Field(description="The duration of the input audio.")]
    text: Annotated[str, Field(description="The translated text.")]
    segments: Annotated[
        Optional[List[TranscriptionSegment]],
        Field(description="Segments of the translated text and their corresponding details."),
    ] = None


class CreateSpeechRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    model: Annotated[
        Union[str, Literal["tts-1", "tts-1-hd"]],
        Field(
            description="One of the available [TTS models](/docs/models/tts): `tts-1` or `tts-1-hd`\n"
        ),
    ]
    input: Annotated[
        str,
        Field(
            description="The text to generate audio for. The maximum length is 4096 characters.",
            max_length=4096,
        ),
    ]
    voice: Annotated[
        Literal["alloy", "echo", "fable", "onyx", "nova", "shimmer"],
        Field(
            description="The voice to use when generating the audio. Supported voices are `alloy`, `echo`, `fable`, `onyx`, `nova`, and `shimmer`. Previews of the voices are available in the [Text to speech guide](/docs/guides/text-to-speech/voice-options)."
        ),
    ]
    response_format: Annotated[
        Optional[Literal["mp3", "opus", "aac", "flac", "wav", "pcm"]],
        Field(
            description="The format to audio in. Supported formats are `mp3`, `opus`, `aac`, `flac`, `wav`, and `pcm`."
        ),
    ] = "mp3"
    speed: Annotated[
        Optional[float],
        Field(
            description="The speed of the generated audio. Select a value from `0.25` to `4.0`. `1.0` is the default.",
            ge=0.25,
            le=4.0,
        ),
    ] = 1.0


class Model(BaseModel):
    id: Annotated[
        str,
        Field(description="The model identifier, which can be referenced in the API endpoints."),
    ]
    created: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) when the model was created."),
    ]
    object: Annotated[
        Literal["model"], Field(description='The object type, which is always "model".')
    ]
    owned_by: Annotated[str, Field(description="The organization that owns the model.")]


class OpenAIFile(BaseModel):
    id: Annotated[
        str,
        Field(description="The file identifier, which can be referenced in the API endpoints."),
    ]
    bytes: Annotated[int, Field(description="The size of the file, in bytes.")]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the file was created."),
    ]
    filename: Annotated[str, Field(description="The name of the file.")]
    object: Annotated[
        Literal["file"], Field(description="The object type, which is always `file`.")
    ]
    purpose: Annotated[
        Literal[
            "assistants",
            "assistants_output",
            "batch",
            "batch_output",
            "fine-tune",
            "fine-tune-results",
            "vision",
        ],
        Field(
            description="The intended purpose of the file. Supported values are `assistants`, `assistants_output`, `batch`, `batch_output`, `fine-tune`, `fine-tune-results` and `vision`."
        ),
    ]
    status: Annotated[
        Literal["uploaded", "processed", "error"],
        Field(
            description="Deprecated. The current status of the file, which can be either `uploaded`, `processed`, or `error`."
        ),
    ]
    status_details: Annotated[
        Optional[str],
        Field(
            description="Deprecated. For details on why a fine-tuning training file failed validation, see the `error` field on `fine_tuning.job`."
        ),
    ] = None


class Upload(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The Upload unique identifier, which can be referenced in API endpoints."
        ),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the Upload was created."),
    ]
    filename: Annotated[str, Field(description="The name of the file to be uploaded.")]
    bytes: Annotated[int, Field(description="The intended number of bytes to be uploaded.")]
    purpose: Annotated[
        str,
        Field(
            description="The intended purpose of the file. [Please refer here](/docs/api-reference/files/object#files/object-purpose) for acceptable values."
        ),
    ]
    status: Annotated[
        Literal["pending", "completed", "cancelled", "expired"],
        Field(description="The status of the Upload."),
    ]
    expires_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the Upload was created."),
    ]
    object: Annotated[
        Optional[Literal["upload"]],
        Field(description='The object type, which is always "upload".'),
    ] = None
    file: Annotated[
        Optional[OpenAIFile],
        Field(description="The ready File object after the Upload is completed."),
    ] = None


class UploadPart(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The upload Part unique identifier, which can be referenced in API endpoints."
        ),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the Part was created."),
    ]
    upload_id: Annotated[
        str,
        Field(description="The ID of the Upload object that this Part was added to."),
    ]
    object: Annotated[
        Literal["upload.part"],
        Field(description="The object type, which is always `upload.part`."),
    ]


class Embedding(BaseModel):
    index: Annotated[
        int, Field(description="The index of the embedding in the list of embeddings.")
    ]
    embedding: Annotated[
        List[float],
        Field(
            description="The embedding vector, which is a list of floats. The length of vector depends on the model as listed in the [embedding guide](/docs/guides/embeddings).\n"
        ),
    ]
    object: Annotated[
        Literal["embedding"],
        Field(description='The object type, which is always "embedding".'),
    ]


class Error1(BaseModel):
    code: Annotated[str, Field(description="A machine-readable error code.")]
    message: Annotated[str, Field(description="A human-readable error message.")]
    param: Annotated[
        str,
        Field(
            description="The parameter that was invalid, usually `training_file` or `validation_file`. This field will be null if the failure was not parameter-specific."
        ),
    ]


class NEpochs1(BaseModel):
    __root__: Annotated[
        int,
        Field(
            description='The number of epochs to train the model for. An epoch refers to one full cycle through the training dataset.\n"auto" decides the optimal number of epochs based on the size of the dataset. If setting the number manually, we support any number between 1 and 50 epochs.',
            ge=1,
            le=50,
        ),
    ]


class Hyperparameters1(BaseModel):
    n_epochs: Annotated[
        Union[Literal["auto"], NEpochs1],
        Field(
            description='The number of epochs to train the model for. An epoch refers to one full cycle through the training dataset.\n"auto" decides the optimal number of epochs based on the size of the dataset. If setting the number manually, we support any number between 1 and 50 epochs.'
        ),
    ]


class FineTuningIntegration(BaseModel):
    type: Annotated[
        Literal["wandb"],
        Field(description="The type of the integration being enabled for the fine-tuning job"),
    ]
    wandb: Annotated[
        Wandb,
        Field(
            description="The settings for your integration with Weights and Biases. This payload specifies the project that\nmetrics will be sent to. Optionally, you can set an explicit display name for your run, add tags\nto your run, and set a default entity (team, username, etc) to be associated with your run.\n"
        ),
    ]


class FineTuningJobEvent(BaseModel):
    id: str
    created_at: int
    level: Literal["info", "warn", "error"]
    message: str
    object: Literal["fine_tuning.job.event"]


class Metrics(BaseModel):
    step: Optional[float] = None
    train_loss: Optional[float] = None
    train_mean_token_accuracy: Optional[float] = None
    valid_loss: Optional[float] = None
    valid_mean_token_accuracy: Optional[float] = None
    full_valid_loss: Optional[float] = None
    full_valid_mean_token_accuracy: Optional[float] = None


class FineTuningJobCheckpoint(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The checkpoint identifier, which can be referenced in the API endpoints."
        ),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the checkpoint was created."),
    ]
    fine_tuned_model_checkpoint: Annotated[
        str,
        Field(description="The name of the fine-tuned checkpoint model that is created."),
    ]
    step_number: Annotated[
        int, Field(description="The step number that the checkpoint was created at.")
    ]
    metrics: Annotated[
        Metrics,
        Field(description="Metrics at the step number during the fine-tuning job."),
    ]
    fine_tuning_job_id: Annotated[
        str,
        Field(description="The name of the fine-tuning job that this checkpoint was created from."),
    ]
    object: Annotated[
        Literal["fine_tuning.job.checkpoint"],
        Field(description='The object type, which is always "fine_tuning.job.checkpoint".'),
    ]


class FinetuneCompletionRequestInput(BaseModel):
    prompt: Annotated[
        Optional[str], Field(description="The input prompt for this training example.")
    ] = None
    completion: Annotated[
        Optional[str],
        Field(description="The desired completion for this training example."),
    ] = None


class CompletionUsage(BaseModel):
    completion_tokens: Annotated[
        int, Field(description="Number of tokens in the generated completion.")
    ]
    prompt_tokens: Annotated[int, Field(description="Number of tokens in the prompt.")]
    total_tokens: Annotated[
        int,
        Field(description="Total number of tokens used in the request (prompt + completion)."),
    ]


class RunCompletionUsage(BaseModel):
    completion_tokens: Annotated[
        int,
        Field(description="Number of completion tokens used over the course of the run."),
    ]
    prompt_tokens: Annotated[
        int,
        Field(description="Number of prompt tokens used over the course of the run."),
    ]
    total_tokens: Annotated[
        int, Field(description="Total number of tokens used (prompt + completion).")
    ]


class RunStepCompletionUsage(BaseModel):
    completion_tokens: Annotated[
        int,
        Field(description="Number of completion tokens used over the course of the run step."),
    ]
    prompt_tokens: Annotated[
        int,
        Field(description="Number of prompt tokens used over the course of the run step."),
    ]
    total_tokens: Annotated[
        int, Field(description="Total number of tokens used (prompt + completion).")
    ]


class AssistantsApiResponseFormatOption(BaseModel):
    __root__: Annotated[
        Union[
            Literal["auto"],
            ResponseFormatText,
            ResponseFormatJsonObject,
            ResponseFormatJsonSchema,
        ],
        Field(
            description='Specifies the format that the model must output. Compatible with [GPT-4o](/docs/models/gpt-4o), [GPT-4 Turbo](/docs/models/gpt-4-turbo-and-gpt-4), and all GPT-3.5 Turbo models since `gpt-3.5-turbo-1106`.\n\nSetting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured Outputs which guarantees the model will match your supplied JSON schema. Learn more in the [Structured Outputs guide](/docs/guides/structured-outputs).\n\nSetting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.\n\n**Important:** when using JSON mode, you **must** also instruct the model to produce JSON yourself via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request. Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length.\n'
        ),
    ]


class CodeInterpreter(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs made available to the `code_interpreter`` tool. There can be a maximum of 20 files associated with the tool.\n",
            max_items=20,
        ),
    ] = []


class FileSearch(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The ID of the [vector store](/docs/api-reference/vector-stores/object) attached to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ] = None


class ToolResources(BaseModel):
    code_interpreter: Optional[CodeInterpreter] = None
    file_search: Optional[FileSearch] = None


class CodeInterpreter1(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs made available to the `code_interpreter` tool. There can be a maximum of 20 files associated with the tool.\n",
            max_items=20,
        ),
    ] = []


class ChunkingStrategy(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["auto"], Field(description="Always `auto`.")]


class Static(BaseModel):
    class Config:
        extra = Extra.forbid

    max_chunk_size_tokens: Annotated[
        int,
        Field(
            description="The maximum number of tokens in each chunk. The default value is `800`. The minimum value is `100` and the maximum value is `4096`.",
            ge=100,
            le=4096,
        ),
    ]
    chunk_overlap_tokens: Annotated[
        int,
        Field(
            description="The number of tokens that overlap between chunks. The default value is `400`.\n\nNote that the overlap must not exceed half of `max_chunk_size_tokens`.\n"
        ),
    ]


class ChunkingStrategy1(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: Static


class VectorStore(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs to add to the vector store. There can be a maximum of 10000 files in a vector store.\n",
            max_items=10000,
        ),
    ] = None
    chunking_strategy: Annotated[
        Optional[Union[ChunkingStrategy, ChunkingStrategy1]],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to a vector store. This can be useful for storing additional information about the vector store in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class FileSearch1(BaseModel):
    vector_store_ids: Annotated[
        List[str],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ]
    vector_stores: Annotated[
        Optional[List[VectorStore]],
        Field(
            description="A helper to create a [vector store](/docs/api-reference/vector-stores/object) with file_ids and attach it to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ] = None


class ChunkingStrategy2(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["auto"], Field(description="Always `auto`.")]


class ChunkingStrategy3(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: Static


class VectorStore1(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs to add to the vector store. There can be a maximum of 10000 files in a vector store.\n",
            max_items=10000,
        ),
    ] = None
    chunking_strategy: Annotated[
        Optional[Union[ChunkingStrategy2, ChunkingStrategy3]],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to a vector store. This can be useful for storing additional information about the vector store in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class FileSearch2(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ] = None
    vector_stores: Annotated[
        List[VectorStore1],
        Field(
            description="A helper to create a [vector store](/docs/api-reference/vector-stores/object) with file_ids and attach it to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ]


class ToolResources1(BaseModel):
    code_interpreter: Optional[CodeInterpreter1] = None
    file_search: Optional[Union[FileSearch1, FileSearch2]] = None


class CodeInterpreter2(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="Overrides the list of [file](/docs/api-reference/files) IDs made available to the `code_interpreter` tool. There can be a maximum of 20 files associated with the tool.\n",
            max_items=20,
        ),
    ] = []


class FileSearch3(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="Overrides the [vector store](/docs/api-reference/vector-stores/object) attached to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ] = None


class ToolResources2(BaseModel):
    code_interpreter: Optional[CodeInterpreter2] = None
    file_search: Optional[FileSearch3] = None


class DeleteAssistantResponse(BaseModel):
    id: str
    deleted: bool
    object: Literal["assistant.deleted"]


class AssistantToolsCode(BaseModel):
    type: Annotated[
        Literal["code_interpreter"],
        Field(description="The type of tool being defined: `code_interpreter`"),
    ]


class FileSearch4(BaseModel):
    max_num_results: Annotated[
        Optional[int],
        Field(
            description="The maximum number of results the file search tool should output. The default is 20 for `gpt-4*` models and 5 for `gpt-3.5-turbo`. This number should be between 1 and 50 inclusive.\n\nNote that the file search tool may output fewer than `max_num_results` results. See the [file search tool documentation](/docs/assistants/tools/file-search/number-of-chunks-returned) for more information.\n",
            ge=1,
            le=50,
        ),
    ] = None


class AssistantToolsFileSearch(BaseModel):
    type: Annotated[
        Literal["file_search"],
        Field(description="The type of tool being defined: `file_search`"),
    ]
    file_search: Annotated[
        Optional[FileSearch4], Field(description="Overrides for the file search tool.")
    ] = None


class AssistantToolsFileSearchTypeOnly(BaseModel):
    type: Annotated[
        Literal["file_search"],
        Field(description="The type of tool being defined: `file_search`"),
    ]


class AssistantToolsFunction(BaseModel):
    type: Annotated[
        Literal["function"],
        Field(description="The type of tool being defined: `function`"),
    ]
    function: FunctionObject


class TruncationObject(BaseModel):
    type: Annotated[
        Literal["auto", "last_messages"],
        Field(
            description="The truncation strategy to use for the thread. The default is `auto`. If set to `last_messages`, the thread will be truncated to the n most recent messages in the thread. When set to `auto`, messages in the middle of the thread will be dropped to fit the context length of the model, `max_prompt_tokens`."
        ),
    ]
    last_messages: Annotated[
        Optional[int],
        Field(
            description="The number of most recent messages from the thread when constructing the context for the run.",
            ge=1,
        ),
    ] = None


class Function3(BaseModel):
    name: Annotated[str, Field(description="The name of the function to call.")]


class AssistantsNamedToolChoice(BaseModel):
    type: Annotated[
        Literal["function", "code_interpreter", "file_search"],
        Field(
            description="The type of the tool. If type is `function`, the function name must be set"
        ),
    ]
    function: Optional[Function3] = None


class LastError(BaseModel):
    code: Annotated[
        Literal["server_error", "rate_limit_exceeded", "invalid_prompt"],
        Field(description="One of `server_error`, `rate_limit_exceeded`, or `invalid_prompt`."),
    ]
    message: Annotated[str, Field(description="A human-readable description of the error.")]


class IncompleteDetails(BaseModel):
    reason: Annotated[
        Optional[Literal["max_completion_tokens", "max_prompt_tokens"]],
        Field(
            description="The reason why the run is incomplete. This will point to which specific token limit was reached over the course of the run."
        ),
    ] = None


class ModifyRunRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class ToolOutput(BaseModel):
    tool_call_id: Annotated[
        Optional[str],
        Field(
            description="The ID of the tool call in the `required_action` object within the run object the output is being submitted for."
        ),
    ] = None
    output: Annotated[
        Optional[str],
        Field(description="The output of the tool call to be submitted to continue the run."),
    ] = None


class SubmitToolOutputsRunRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    tool_outputs: Annotated[
        List[ToolOutput],
        Field(description="A list of tools for which the outputs are being submitted."),
    ]
    stream: Annotated[
        Optional[bool],
        Field(
            description="If `true`, returns a stream of events that happen during the Run as server-sent events, terminating when the Run enters a terminal state with a `data: [DONE]` message.\n"
        ),
    ] = None


class Function4(BaseModel):
    name: Annotated[str, Field(description="The name of the function.")]
    arguments: Annotated[
        str,
        Field(description="The arguments that the model expects you to pass to the function."),
    ]


class RunToolCallObject(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The ID of the tool call. This ID must be referenced when you submit the tool outputs in using the [Submit tool outputs to run](/docs/api-reference/runs/submitToolOutputs) endpoint."
        ),
    ]
    type: Annotated[
        Literal["function"],
        Field(
            description="The type of tool call the output is required for. For now, this is always `function`."
        ),
    ]
    function: Annotated[Function4, Field(description="The function definition.")]


class CodeInterpreter3(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs made available to the `code_interpreter` tool. There can be a maximum of 20 files associated with the tool.\n",
            max_items=20,
        ),
    ] = []


class FileSearch5(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The ID of the [vector store](/docs/api-reference/vector-stores/object) attached to this assistant. There can be a maximum of 1 vector store attached to the assistant.\n",
            max_items=1,
        ),
    ] = None


class ToolResources3(BaseModel):
    code_interpreter: Optional[CodeInterpreter3] = None
    file_search: Optional[FileSearch5] = None


class FileSearch6(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ] = None


class ToolResources4(BaseModel):
    code_interpreter: Optional[CodeInterpreter3] = None
    file_search: Optional[FileSearch6] = None


class ThreadObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["thread"],
        Field(description="The object type, which is always `thread`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the thread was created."),
    ]
    tool_resources: Annotated[
        ToolResources4,
        Field(
            description="A set of resources that are made available to the assistant's tools in this thread. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ]
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]


class ChunkingStrategy4(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["auto"], Field(description="Always `auto`.")]


class ChunkingStrategy5(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: Static


class VectorStore2(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs to add to the vector store. There can be a maximum of 10000 files in a vector store.\n",
            max_items=10000,
        ),
    ] = None
    chunking_strategy: Annotated[
        Optional[Union[ChunkingStrategy4, ChunkingStrategy5]],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to a vector store. This can be useful for storing additional information about the vector store in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class FileSearch7(BaseModel):
    vector_store_ids: Annotated[
        List[str],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ]
    vector_stores: Annotated[
        Optional[List[VectorStore2]],
        Field(
            description="A helper to create a [vector store](/docs/api-reference/vector-stores/object) with file_ids and attach it to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ] = None


class ChunkingStrategy6(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["auto"], Field(description="Always `auto`.")]


class ChunkingStrategy7(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: Static


class VectorStore3(BaseModel):
    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [file](/docs/api-reference/files) IDs to add to the vector store. There can be a maximum of 10000 files in a vector store.\n",
            max_items=10000,
        ),
    ] = None
    chunking_strategy: Annotated[
        Optional[Union[ChunkingStrategy6, ChunkingStrategy7]],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to a vector store. This can be useful for storing additional information about the vector store in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class FileSearch8(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ] = None
    vector_stores: Annotated[
        List[VectorStore3],
        Field(
            description="A helper to create a [vector store](/docs/api-reference/vector-stores/object) with file_ids and attach it to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ]


class ToolResources5(BaseModel):
    code_interpreter: Optional[CodeInterpreter3] = None
    file_search: Optional[Union[FileSearch7, FileSearch8]] = None


class FileSearch9(BaseModel):
    vector_store_ids: Annotated[
        Optional[List[str]],
        Field(
            description="The [vector store](/docs/api-reference/vector-stores/object) attached to this thread. There can be a maximum of 1 vector store attached to the thread.\n",
            max_items=1,
        ),
    ] = None


class ToolResources6(BaseModel):
    code_interpreter: Optional[CodeInterpreter3] = None
    file_search: Optional[FileSearch9] = None


class ModifyThreadRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    tool_resources: Annotated[
        Optional[ToolResources6],
        Field(
            description="A set of resources that are made available to the assistant's tools in this thread. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class DeleteThreadResponse(BaseModel):
    id: str
    deleted: bool
    object: Literal["thread.deleted"]


class ListThreadsResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[ThreadObject]
    first_id: Annotated[str, Field(example="asst_abc123")]
    last_id: Annotated[str, Field(example="asst_abc456")]
    has_more: Annotated[bool, Field(example=False)]


class IncompleteDetails1(BaseModel):
    reason: Annotated[
        Literal["content_filter", "max_tokens", "run_cancelled", "run_expired", "run_failed"],
        Field(description="The reason the message is incomplete."),
    ]


class Attachment(BaseModel):
    file_id: Annotated[
        Optional[str], Field(description="The ID of the file to attach to the message.")
    ] = None
    tools: Annotated[
        Optional[List[Union[AssistantToolsCode, AssistantToolsFileSearchTypeOnly]]],
        Field(description="The tools to add this file to."),
    ] = None


class ModifyMessageRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class DeleteMessageResponse(BaseModel):
    id: str
    deleted: bool
    object: Literal["thread.message.deleted"]


class ImageFile(BaseModel):
    file_id: Annotated[
        str,
        Field(
            description='The [File](/docs/api-reference/files) ID of the image in the message content. Set `purpose="vision"` when uploading the File if you need to later display the file content.'
        ),
    ]
    detail: Annotated[
        Optional[Literal["auto", "low", "high"]],
        Field(
            description="Specifies the detail level of the image if specified by the user. `low` uses fewer tokens, you can opt in to high resolution using `high`."
        ),
    ] = "auto"


class MessageContentImageFileObject(BaseModel):
    type: Annotated[Literal["image_file"], Field(description="Always `image_file`.")]
    image_file: ImageFile


class ImageFile1(BaseModel):
    file_id: Annotated[
        Optional[str],
        Field(
            description='The [File](/docs/api-reference/files) ID of the image in the message content. Set `purpose="vision"` when uploading the File if you need to later display the file content.'
        ),
    ] = None
    detail: Annotated[
        Optional[Literal["auto", "low", "high"]],
        Field(
            description="Specifies the detail level of the image if specified by the user. `low` uses fewer tokens, you can opt in to high resolution using `high`."
        ),
    ] = "auto"


class MessageDeltaContentImageFileObject(BaseModel):
    index: Annotated[int, Field(description="The index of the content part in the message.")]
    type: Annotated[Literal["image_file"], Field(description="Always `image_file`.")]
    image_file: Optional[ImageFile1] = None


class ImageUrl1(BaseModel):
    url: Annotated[
        AnyUrl,
        Field(
            description="The external URL of the image, must be a supported image types: jpeg, jpg, png, gif, webp."
        ),
    ]
    detail: Annotated[
        Optional[Literal["auto", "low", "high"]],
        Field(
            description="Specifies the detail level of the image. `low` uses fewer tokens, you can opt in to high resolution using `high`. Default value is `auto`"
        ),
    ] = "auto"


class MessageContentImageUrlObject(BaseModel):
    type: Annotated[Literal["image_url"], Field(description="The type of the content part.")]
    image_url: ImageUrl1


class ImageUrl2(BaseModel):
    url: Annotated[
        Optional[str],
        Field(
            description="The URL of the image, must be a supported image types: jpeg, jpg, png, gif, webp."
        ),
    ] = None
    detail: Annotated[
        Optional[Literal["auto", "low", "high"]],
        Field(
            description="Specifies the detail level of the image. `low` uses fewer tokens, you can opt in to high resolution using `high`."
        ),
    ] = "auto"


class MessageDeltaContentImageUrlObject(BaseModel):
    index: Annotated[int, Field(description="The index of the content part in the message.")]
    type: Annotated[Literal["image_url"], Field(description="Always `image_url`.")]
    image_url: Optional[ImageUrl2] = None


class MessageContentRefusalObject(BaseModel):
    type: Annotated[Literal["refusal"], Field(description="Always `refusal`.")]
    refusal: str


class MessageRequestContentTextObject(BaseModel):
    type: Annotated[Literal["text"], Field(description="Always `text`.")]
    text: Annotated[str, Field(description="Text content to be sent to the model")]


class FileCitation(BaseModel):
    file_id: Annotated[str, Field(description="The ID of the specific File the citation is from.")]


class MessageContentTextAnnotationsFileCitationObject(BaseModel):
    type: Annotated[Literal["file_citation"], Field(description="Always `file_citation`.")]
    text: Annotated[
        str,
        Field(description="The text in the message content that needs to be replaced."),
    ]
    file_citation: FileCitation
    start_index: Annotated[int, Field(ge=0)]
    end_index: Annotated[int, Field(ge=0)]


class FilePath(BaseModel):
    file_id: Annotated[str, Field(description="The ID of the file that was generated.")]


class MessageContentTextAnnotationsFilePathObject(BaseModel):
    type: Annotated[Literal["file_path"], Field(description="Always `file_path`.")]
    text: Annotated[
        str,
        Field(description="The text in the message content that needs to be replaced."),
    ]
    file_path: FilePath
    start_index: Annotated[int, Field(ge=0)]
    end_index: Annotated[int, Field(ge=0)]


class MessageDeltaContentRefusalObject(BaseModel):
    index: Annotated[int, Field(description="The index of the refusal part in the message.")]
    type: Annotated[Literal["refusal"], Field(description="Always `refusal`.")]
    refusal: Optional[str] = None


class FileCitation1(BaseModel):
    file_id: Annotated[
        Optional[str],
        Field(description="The ID of the specific File the citation is from."),
    ] = None
    quote: Annotated[Optional[str], Field(description="The specific quote in the file.")] = None


class MessageDeltaContentTextAnnotationsFileCitationObject(BaseModel):
    index: Annotated[
        int, Field(description="The index of the annotation in the text content part.")
    ]
    type: Annotated[Literal["file_citation"], Field(description="Always `file_citation`.")]
    text: Annotated[
        Optional[str],
        Field(description="The text in the message content that needs to be replaced."),
    ] = None
    file_citation: Optional[FileCitation1] = None
    start_index: Annotated[Optional[int], Field(ge=0)] = None
    end_index: Annotated[Optional[int], Field(ge=0)] = None


class FilePath1(BaseModel):
    file_id: Annotated[
        Optional[str], Field(description="The ID of the file that was generated.")
    ] = None


class MessageDeltaContentTextAnnotationsFilePathObject(BaseModel):
    index: Annotated[
        int, Field(description="The index of the annotation in the text content part.")
    ]
    type: Annotated[Literal["file_path"], Field(description="Always `file_path`.")]
    text: Annotated[
        Optional[str],
        Field(description="The text in the message content that needs to be replaced."),
    ] = None
    file_path: Optional[FilePath1] = None
    start_index: Annotated[Optional[int], Field(ge=0)] = None
    end_index: Annotated[Optional[int], Field(ge=0)] = None


class LastError1(BaseModel):
    code: Annotated[
        Literal["server_error", "rate_limit_exceeded"],
        Field(description="One of `server_error` or `rate_limit_exceeded`."),
    ]
    message: Annotated[str, Field(description="A human-readable description of the error.")]


class MessageCreation(BaseModel):
    message_id: Annotated[
        str,
        Field(description="The ID of the message that was created by this run step."),
    ]


class RunStepDetailsMessageCreationObject(BaseModel):
    type: Annotated[Literal["message_creation"], Field(description="Always `message_creation`.")]
    message_creation: MessageCreation


class MessageCreation1(BaseModel):
    message_id: Annotated[
        Optional[str],
        Field(description="The ID of the message that was created by this run step."),
    ] = None


class RunStepDeltaStepDetailsMessageCreationObject(BaseModel):
    type: Annotated[Literal["message_creation"], Field(description="Always `message_creation`.")]
    message_creation: Optional[MessageCreation1] = None


class RunStepDetailsToolCallsCodeOutputLogsObject(BaseModel):
    type: Annotated[Literal["logs"], Field(description="Always `logs`.")]
    logs: Annotated[str, Field(description="The text output from the Code Interpreter tool call.")]


class RunStepDeltaStepDetailsToolCallsCodeOutputLogsObject(BaseModel):
    index: Annotated[int, Field(description="The index of the output in the outputs array.")]
    type: Annotated[Literal["logs"], Field(description="Always `logs`.")]
    logs: Annotated[
        Optional[str],
        Field(description="The text output from the Code Interpreter tool call."),
    ] = None


class Image1(BaseModel):
    file_id: Annotated[
        str, Field(description="The [file](/docs/api-reference/files) ID of the image.")
    ]


class RunStepDetailsToolCallsCodeOutputImageObject(BaseModel):
    type: Annotated[Literal["image"], Field(description="Always `image`.")]
    image: Image1


class Image2(BaseModel):
    file_id: Annotated[
        Optional[str],
        Field(description="The [file](/docs/api-reference/files) ID of the image."),
    ] = None


class RunStepDeltaStepDetailsToolCallsCodeOutputImageObject(BaseModel):
    index: Annotated[int, Field(description="The index of the output in the outputs array.")]
    type: Annotated[Literal["image"], Field(description="Always `image`.")]
    image: Optional[Image2] = None


class RunStepDetailsToolCallsFileSearchObject(BaseModel):
    id: Annotated[str, Field(description="The ID of the tool call object.")]
    type: Annotated[
        Literal["file_search"],
        Field(
            description="The type of tool call. This is always going to be `file_search` for this type of tool call."
        ),
    ]
    file_search: Annotated[
        Dict[str, Any],
        Field(description="For now, this is always going to be an empty object."),
    ]


class RunStepDeltaStepDetailsToolCallsFileSearchObject(BaseModel):
    index: Annotated[int, Field(description="The index of the tool call in the tool calls array.")]
    id: Annotated[Optional[str], Field(description="The ID of the tool call object.")] = None
    type: Annotated[
        Literal["file_search"],
        Field(
            description="The type of tool call. This is always going to be `file_search` for this type of tool call."
        ),
    ]
    file_search: Annotated[
        Dict[str, Any],
        Field(description="For now, this is always going to be an empty object."),
    ]


class Function5(BaseModel):
    name: Annotated[str, Field(description="The name of the function.")]
    arguments: Annotated[str, Field(description="The arguments passed to the function.")]
    output: Annotated[
        str,
        Field(
            description="The output of the function. This will be `null` if the outputs have not been [submitted](/docs/api-reference/runs/submitToolOutputs) yet."
        ),
    ]


class RunStepDetailsToolCallsFunctionObject(BaseModel):
    id: Annotated[str, Field(description="The ID of the tool call object.")]
    type: Annotated[
        Literal["function"],
        Field(
            description="The type of tool call. This is always going to be `function` for this type of tool call."
        ),
    ]
    function: Annotated[
        Function5, Field(description="The definition of the function that was called.")
    ]


class Function6(BaseModel):
    name: Annotated[Optional[str], Field(description="The name of the function.")] = None
    arguments: Annotated[
        Optional[str], Field(description="The arguments passed to the function.")
    ] = None
    output: Annotated[
        Optional[str],
        Field(
            description="The output of the function. This will be `null` if the outputs have not been [submitted](/docs/api-reference/runs/submitToolOutputs) yet."
        ),
    ] = None


class RunStepDeltaStepDetailsToolCallsFunctionObject(BaseModel):
    index: Annotated[int, Field(description="The index of the tool call in the tool calls array.")]
    id: Annotated[Optional[str], Field(description="The ID of the tool call object.")] = None
    type: Annotated[
        Literal["function"],
        Field(
            description="The type of tool call. This is always going to be `function` for this type of tool call."
        ),
    ]
    function: Annotated[
        Optional[Function6],
        Field(description="The definition of the function that was called."),
    ] = None


class VectorStoreExpirationAfter(BaseModel):
    anchor: Annotated[
        Literal["last_active_at"],
        Field(
            description="Anchor timestamp after which the expiration policy applies. Supported anchors: `last_active_at`."
        ),
    ]
    days: Annotated[
        int,
        Field(
            description="The number of days after the anchor time that the vector store will expire.",
            ge=1,
            le=365,
        ),
    ]


class FileCounts(BaseModel):
    in_progress: Annotated[
        int,
        Field(description="The number of files that are currently being processed."),
    ]
    completed: Annotated[
        int,
        Field(description="The number of files that have been successfully processed."),
    ]
    failed: Annotated[int, Field(description="The number of files that have failed to process.")]
    cancelled: Annotated[int, Field(description="The number of files that were cancelled.")]
    total: Annotated[int, Field(description="The total number of files.")]


class VectorStoreObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["vector_store"],
        Field(description="The object type, which is always `vector_store`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the vector store was created."),
    ]
    name: Annotated[str, Field(description="The name of the vector store.")]
    usage_bytes: Annotated[
        int,
        Field(description="The total number of bytes used by the files in the vector store."),
    ]
    file_counts: FileCounts
    status: Annotated[
        Literal["expired", "in_progress", "completed"],
        Field(
            description="The status of the vector store, which can be either `expired`, `in_progress`, or `completed`. A status of `completed` indicates that the vector store is ready for use."
        ),
    ]
    expires_after: Optional[VectorStoreExpirationAfter] = None
    expires_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the vector store will expire."),
    ] = None
    last_active_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the vector store was last active."
        ),
    ]
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]


class UpdateVectorStoreRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    name: Annotated[Optional[str], Field(description="The name of the vector store.")] = None
    expires_after: Optional[VectorStoreExpirationAfter] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class ListVectorStoresResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[VectorStoreObject]
    first_id: Annotated[str, Field(example="vs_abc123")]
    last_id: Annotated[str, Field(example="vs_abc456")]
    has_more: Annotated[bool, Field(example=False)]


class DeleteVectorStoreResponse(BaseModel):
    id: str
    deleted: bool
    object: Literal["vector_store.deleted"]


class LastError2(BaseModel):
    code: Annotated[
        Literal["server_error", "unsupported_file", "invalid_file"],
        Field(description="One of `server_error` or `rate_limit_exceeded`."),
    ]
    message: Annotated[str, Field(description="A human-readable description of the error.")]


class OtherChunkingStrategyResponseParam(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["other"], Field(description="Always `other`.")]


class StaticChunkingStrategy(BaseModel):
    class Config:
        extra = Extra.forbid

    max_chunk_size_tokens: Annotated[
        int,
        Field(
            description="The maximum number of tokens in each chunk. The default value is `800`. The minimum value is `100` and the maximum value is `4096`.",
            ge=100,
            le=4096,
        ),
    ]
    chunk_overlap_tokens: Annotated[
        int,
        Field(
            description="The number of tokens that overlap between chunks. The default value is `400`.\n\nNote that the overlap must not exceed half of `max_chunk_size_tokens`.\n"
        ),
    ]


class AutoChunkingStrategyRequestParam(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["auto"], Field(description="Always `auto`.")]


class StaticChunkingStrategyRequestParam(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: StaticChunkingStrategy


class ChunkingStrategyRequestParam(BaseModel):
    __root__: Annotated[
        Union[AutoChunkingStrategyRequestParam, StaticChunkingStrategyRequestParam],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy."
        ),
    ]


class CreateVectorStoreFileRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file_id: Annotated[
        str,
        Field(
            description="A [File](/docs/api-reference/files) ID that the vector store should use. Useful for tools like `file_search` that can access files."
        ),
    ]
    chunking_strategy: Optional[ChunkingStrategyRequestParam] = None


class DeleteVectorStoreFileResponse(BaseModel):
    id: str
    deleted: bool
    object: Literal["vector_store.file.deleted"]


class FileCounts1(BaseModel):
    in_progress: Annotated[
        int,
        Field(description="The number of files that are currently being processed."),
    ]
    completed: Annotated[int, Field(description="The number of files that have been processed.")]
    failed: Annotated[int, Field(description="The number of files that have failed to process.")]
    cancelled: Annotated[int, Field(description="The number of files that where cancelled.")]
    total: Annotated[int, Field(description="The total number of files.")]


class VectorStoreFileBatchObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["vector_store.files_batch"],
        Field(description="The object type, which is always `vector_store.file_batch`."),
    ]
    created_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the vector store files batch was created."
        ),
    ]
    vector_store_id: Annotated[
        str,
        Field(
            description="The ID of the [vector store](/docs/api-reference/vector-stores/object) that the [File](/docs/api-reference/files) is attached to."
        ),
    ]
    status: Annotated[
        Literal["in_progress", "completed", "cancelled", "failed"],
        Field(
            description="The status of the vector store files batch, which can be either `in_progress`, `completed`, `cancelled` or `failed`."
        ),
    ]
    file_counts: FileCounts1


class CreateVectorStoreFileBatchRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file_ids: Annotated[
        List[str],
        Field(
            description="A list of [File](/docs/api-reference/files) IDs that the vector store should use. Useful for tools like `file_search` that can access files.",
            max_items=500,
            min_items=1,
        ),
    ]
    chunking_strategy: Optional[ChunkingStrategyRequestParam] = None


class ThreadStreamEvent1(BaseModel):
    event: Literal["thread.created"]
    data: ThreadObject


class ThreadStreamEvent(BaseModel):
    __root__: ThreadStreamEvent1


class ErrorEvent(BaseModel):
    event: Literal["error"]
    data: Error


class DoneEvent(BaseModel):
    event: Literal["done"]
    data: Literal["[DONE]"]


class Datum(BaseModel):
    code: Annotated[
        Optional[str], Field(description="An error code identifying the error type.")
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human-readable message providing more details about the error."),
    ] = None
    param: Annotated[
        Optional[str],
        Field(description="The name of the parameter that caused the error, if applicable."),
    ] = None
    line: Annotated[
        Optional[int],
        Field(
            description="The line number of the input file where the error occurred, if applicable."
        ),
    ] = None


class Errors(BaseModel):
    object: Annotated[
        Optional[str], Field(description="The object type, which is always `list`.")
    ] = None
    data: Optional[List[Datum]] = None


class RequestCounts(BaseModel):
    total: Annotated[int, Field(description="Total number of requests in the batch.")]
    completed: Annotated[
        int,
        Field(description="Number of requests that have been completed successfully."),
    ]
    failed: Annotated[int, Field(description="Number of requests that have failed.")]


class Batch(BaseModel):
    id: str
    object: Annotated[
        Literal["batch"], Field(description="The object type, which is always `batch`.")
    ]
    endpoint: Annotated[str, Field(description="The OpenAI API endpoint used by the batch.")]
    errors: Optional[Errors] = None
    input_file_id: Annotated[str, Field(description="The ID of the input file for the batch.")]
    completion_window: Annotated[
        str,
        Field(description="The time frame within which the batch should be processed."),
    ]
    status: Annotated[
        Literal[
            "validating",
            "failed",
            "in_progress",
            "finalizing",
            "completed",
            "expired",
            "cancelling",
            "cancelled",
        ],
        Field(description="The current status of the batch."),
    ]
    output_file_id: Annotated[
        Optional[str],
        Field(
            description="The ID of the file containing the outputs of successfully executed requests."
        ),
    ] = None
    error_file_id: Annotated[
        Optional[str],
        Field(description="The ID of the file containing the outputs of requests with errors."),
    ] = None
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the batch was created."),
    ]
    in_progress_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch started processing."),
    ] = None
    expires_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch will expire."),
    ] = None
    finalizing_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch started finalizing."),
    ] = None
    completed_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch was completed."),
    ] = None
    failed_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch failed."),
    ] = None
    expired_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch expired."),
    ] = None
    cancelling_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch started cancelling."),
    ] = None
    cancelled_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) for when the batch was cancelled."),
    ] = None
    request_counts: Annotated[
        Optional[RequestCounts],
        Field(description="The request counts for different statuses within the batch."),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class BatchRequestInput(BaseModel):
    custom_id: Annotated[
        Optional[str],
        Field(
            description="A developer-provided per-request id that will be used to match outputs to inputs. Must be unique for each request in a batch."
        ),
    ] = None
    method: Annotated[
        Optional[Literal["POST"]],
        Field(
            description="The HTTP method to be used for the request. Currently only `POST` is supported."
        ),
    ] = None
    url: Annotated[
        Optional[str],
        Field(
            description="The OpenAI API relative URL to be used for the request. Currently `/v1/chat/completions`, `/v1/embeddings`, and `/v1/completions` are supported."
        ),
    ] = None


class Response(BaseModel):
    status_code: Annotated[
        Optional[int], Field(description="The HTTP status code of the response")
    ] = None
    request_id: Annotated[
        Optional[str],
        Field(
            description="An unique identifier for the OpenAI API request. Please include this request ID when contacting support."
        ),
    ] = None
    body: Annotated[
        Optional[Dict[str, Any]], Field(description="The JSON body of the response")
    ] = None


class Error2(BaseModel):
    code: Annotated[Optional[str], Field(description="A machine-readable error code.")] = None
    message: Annotated[Optional[str], Field(description="A human-readable error message.")] = None


class BatchRequestOutput(BaseModel):
    id: Optional[str] = None
    custom_id: Annotated[
        Optional[str],
        Field(
            description="A developer-provided per-request id that will be used to match outputs to inputs."
        ),
    ] = None
    response: Optional[Response] = None
    error: Annotated[
        Optional[Error2],
        Field(
            description="For requests that failed with a non-HTTP error, this will contain more information on the cause of the failure."
        ),
    ] = None


class ListBatchesResponse(BaseModel):
    data: List[Batch]
    first_id: Annotated[Optional[str], Field(example="batch_abc123")] = None
    last_id: Annotated[Optional[str], Field(example="batch_abc456")] = None
    has_more: bool
    object: Literal["list"]


class AuditLogActorServiceAccount(BaseModel):
    id: Annotated[Optional[str], Field(description="The service account id.")] = None


class AuditLogActorUser(BaseModel):
    id: Annotated[Optional[str], Field(description="The user id.")] = None
    email: Annotated[Optional[str], Field(description="The user email.")] = None


class AuditLogActorApiKey(BaseModel):
    id: Annotated[Optional[str], Field(description="The tracking id of the API key.")] = None
    type: Annotated[
        Optional[Literal["user", "service_account"]],
        Field(description="The type of API key. Can be either `user` or `service_account`."),
    ] = None
    user: Optional[AuditLogActorUser] = None
    service_account: Optional[AuditLogActorServiceAccount] = None


class AuditLogActorSession(BaseModel):
    user: Optional[AuditLogActorUser] = None
    ip_address: Annotated[
        Optional[str],
        Field(description="The IP address from which the action was performed."),
    ] = None


class AuditLogActor(BaseModel):
    type: Annotated[
        Optional[Literal["session", "api_key"]],
        Field(description="The type of actor. Is either `session` or `api_key`."),
    ] = None
    session: Optional[AuditLogActorSession] = None
    api_key: Optional[AuditLogActorApiKey] = None


class AuditLogEventType(BaseModel):
    __root__: Annotated[
        Literal[
            "api_key.created",
            "api_key.updated",
            "api_key.deleted",
            "invite.sent",
            "invite.accepted",
            "invite.deleted",
            "login.succeeded",
            "login.failed",
            "logout.succeeded",
            "logout.failed",
            "organization.updated",
            "project.created",
            "project.updated",
            "project.archived",
            "service_account.created",
            "service_account.updated",
            "service_account.deleted",
            "user.added",
            "user.updated",
            "user.deleted",
        ],
        Field(description="The event type."),
    ]


class Project(BaseModel):
    id: Annotated[Optional[str], Field(description="The project ID.")] = None
    name: Annotated[Optional[str], Field(description="The project title.")] = None


class Data(BaseModel):
    scopes: Annotated[
        Optional[List[str]],
        Field(description='A list of scopes allowed for the API key, e.g. `["api.model.request"]`'),
    ] = None


class ApiKeyCreated(BaseModel):
    id: Annotated[Optional[str], Field(description="The tracking ID of the API key.")] = None
    data: Annotated[
        Optional[Data], Field(description="The payload used to create the API key.")
    ] = None


class ChangesRequested(BaseModel):
    scopes: Annotated[
        Optional[List[str]],
        Field(description='A list of scopes allowed for the API key, e.g. `["api.model.request"]`'),
    ] = None


class ApiKeyUpdated(BaseModel):
    id: Annotated[Optional[str], Field(description="The tracking ID of the API key.")] = None
    changes_requested: Annotated[
        Optional[ChangesRequested],
        Field(description="The payload used to update the API key."),
    ] = None


class ApiKeyDeleted(BaseModel):
    id: Annotated[Optional[str], Field(description="The tracking ID of the API key.")] = None


class Data1(BaseModel):
    email: Annotated[
        Optional[str], Field(description="The email invited to the organization.")
    ] = None
    role: Annotated[
        Optional[str],
        Field(description="The role the email was invited to be. Is either `owner` or `member`."),
    ] = None


class InviteSent(BaseModel):
    id: Annotated[Optional[str], Field(description="The ID of the invite.")] = None
    data: Annotated[
        Optional[Data1], Field(description="The payload used to create the invite.")
    ] = None


class InviteAccepted(BaseModel):
    id: Annotated[Optional[str], Field(description="The ID of the invite.")] = None


class InviteDeleted(BaseModel):
    id: Annotated[Optional[str], Field(description="The ID of the invite.")] = None


class LoginFailed(BaseModel):
    error_code: Annotated[Optional[str], Field(description="The error code of the failure.")] = None
    error_message: Annotated[
        Optional[str], Field(description="The error message of the failure.")
    ] = None


class LogoutFailed(BaseModel):
    error_code: Annotated[Optional[str], Field(description="The error code of the failure.")] = None
    error_message: Annotated[
        Optional[str], Field(description="The error message of the failure.")
    ] = None


class Settings(BaseModel):
    threads_ui_visibility: Annotated[
        Optional[str],
        Field(
            description="Visibility of the threads page which shows messages created with the Assistants API and Playground. One of `ANY_ROLE`, `OWNERS`, or `NONE`."
        ),
    ] = None
    usage_dashboard_visibility: Annotated[
        Optional[str],
        Field(
            description="Visibility of the usage dashboard which shows activity and costs for your organization. One of `ANY_ROLE` or `OWNERS`."
        ),
    ] = None


class ChangesRequested1(BaseModel):
    title: Annotated[Optional[str], Field(description="The organization title.")] = None
    description: Annotated[Optional[str], Field(description="The organization description.")] = None
    name: Annotated[Optional[str], Field(description="The organization name.")] = None
    settings: Optional[Settings] = None


class OrganizationUpdated(BaseModel):
    id: Annotated[Optional[str], Field(description="The organization ID.")] = None
    changes_requested: Annotated[
        Optional[ChangesRequested1],
        Field(description="The payload used to update the organization settings."),
    ] = None


class Data2(BaseModel):
    name: Annotated[Optional[str], Field(description="The project name.")] = None
    title: Annotated[
        Optional[str],
        Field(description="The title of the project as seen on the dashboard."),
    ] = None


class ProjectCreated(BaseModel):
    id: Annotated[Optional[str], Field(description="The project ID.")] = None
    data: Annotated[
        Optional[Data2], Field(description="The payload used to create the project.")
    ] = None


class ChangesRequested2(BaseModel):
    title: Annotated[
        Optional[str],
        Field(description="The title of the project as seen on the dashboard."),
    ] = None


class ProjectUpdated(BaseModel):
    id: Annotated[Optional[str], Field(description="The project ID.")] = None
    changes_requested: Annotated[
        Optional[ChangesRequested2],
        Field(description="The payload used to update the project."),
    ] = None


class ProjectArchived(BaseModel):
    id: Annotated[Optional[str], Field(description="The project ID.")] = None


class Data3(BaseModel):
    role: Annotated[
        Optional[str],
        Field(description="The role of the service account. Is either `owner` or `member`."),
    ] = None


class ServiceAccountCreated(BaseModel):
    id: Annotated[Optional[str], Field(description="The service account ID.")] = None
    data: Annotated[
        Optional[Data3],
        Field(description="The payload used to create the service account."),
    ] = None


class ChangesRequested3(BaseModel):
    role: Annotated[
        Optional[str],
        Field(description="The role of the service account. Is either `owner` or `member`."),
    ] = None


class ServiceAccountUpdated(BaseModel):
    id: Annotated[Optional[str], Field(description="The service account ID.")] = None
    changes_requested: Annotated[
        Optional[ChangesRequested3],
        Field(description="The payload used to updated the service account."),
    ] = None


class ServiceAccountDeleted(BaseModel):
    id: Annotated[Optional[str], Field(description="The service account ID.")] = None


class Data4(BaseModel):
    role: Annotated[
        Optional[str],
        Field(description="The role of the user. Is either `owner` or `member`."),
    ] = None


class UserAdded(BaseModel):
    id: Annotated[Optional[str], Field(description="The user ID.")] = None
    data: Annotated[
        Optional[Data4],
        Field(description="The payload used to add the user to the project."),
    ] = None


class ChangesRequested4(BaseModel):
    role: Annotated[
        Optional[str],
        Field(description="The role of the user. Is either `owner` or `member`."),
    ] = None


class UserUpdated(BaseModel):
    id: Annotated[Optional[str], Field(description="The project ID.")] = None
    changes_requested: Annotated[
        Optional[ChangesRequested4],
        Field(description="The payload used to update the user."),
    ] = None


class UserDeleted(BaseModel):
    id: Annotated[Optional[str], Field(description="The user ID.")] = None


class AuditLog(BaseModel):
    id: Annotated[str, Field(description="The ID of this log.")]
    type: AuditLogEventType
    effective_at: Annotated[int, Field(description="The Unix timestamp (in seconds) of the event.")]
    project: Annotated[
        Optional[Project],
        Field(
            description="The project that the action was scoped to. Absent for actions not scoped to projects."
        ),
    ] = None
    actor: AuditLogActor
    api_key_created: Annotated[
        Optional[ApiKeyCreated],
        Field(
            alias="api_key.created",
            description="The details for events with this `type`.",
        ),
    ] = None
    api_key_updated: Annotated[
        Optional[ApiKeyUpdated],
        Field(
            alias="api_key.updated",
            description="The details for events with this `type`.",
        ),
    ] = None
    api_key_deleted: Annotated[
        Optional[ApiKeyDeleted],
        Field(
            alias="api_key.deleted",
            description="The details for events with this `type`.",
        ),
    ] = None
    invite_sent: Annotated[
        Optional[InviteSent],
        Field(alias="invite.sent", description="The details for events with this `type`."),
    ] = None
    invite_accepted: Annotated[
        Optional[InviteAccepted],
        Field(
            alias="invite.accepted",
            description="The details for events with this `type`.",
        ),
    ] = None
    invite_deleted: Annotated[
        Optional[InviteDeleted],
        Field(
            alias="invite.deleted",
            description="The details for events with this `type`.",
        ),
    ] = None
    login_failed: Annotated[
        Optional[LoginFailed],
        Field(alias="login.failed", description="The details for events with this `type`."),
    ] = None
    logout_failed: Annotated[
        Optional[LogoutFailed],
        Field(
            alias="logout.failed",
            description="The details for events with this `type`.",
        ),
    ] = None
    organization_updated: Annotated[
        Optional[OrganizationUpdated],
        Field(
            alias="organization.updated",
            description="The details for events with this `type`.",
        ),
    ] = None
    project_created: Annotated[
        Optional[ProjectCreated],
        Field(
            alias="project.created",
            description="The details for events with this `type`.",
        ),
    ] = None
    project_updated: Annotated[
        Optional[ProjectUpdated],
        Field(
            alias="project.updated",
            description="The details for events with this `type`.",
        ),
    ] = None
    project_archived: Annotated[
        Optional[ProjectArchived],
        Field(
            alias="project.archived",
            description="The details for events with this `type`.",
        ),
    ] = None
    service_account_created: Annotated[
        Optional[ServiceAccountCreated],
        Field(
            alias="service_account.created",
            description="The details for events with this `type`.",
        ),
    ] = None
    service_account_updated: Annotated[
        Optional[ServiceAccountUpdated],
        Field(
            alias="service_account.updated",
            description="The details for events with this `type`.",
        ),
    ] = None
    service_account_deleted: Annotated[
        Optional[ServiceAccountDeleted],
        Field(
            alias="service_account.deleted",
            description="The details for events with this `type`.",
        ),
    ] = None
    user_added: Annotated[
        Optional[UserAdded],
        Field(alias="user.added", description="The details for events with this `type`."),
    ] = None
    user_updated: Annotated[
        Optional[UserUpdated],
        Field(alias="user.updated", description="The details for events with this `type`."),
    ] = None
    user_deleted: Annotated[
        Optional[UserDeleted],
        Field(alias="user.deleted", description="The details for events with this `type`."),
    ] = None


class ListAuditLogsResponse(BaseModel):
    object: Literal["list"]
    data: List[AuditLog]
    first_id: Annotated[str, Field(example="audit_log-defb456h8dks")]
    last_id: Annotated[str, Field(example="audit_log-hnbkd8s93s")]
    has_more: bool


class Invite(BaseModel):
    object: Annotated[
        Literal["organization.invite"],
        Field(description="The object type, which is always `organization.invite`"),
    ]
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    email: Annotated[
        str,
        Field(description="The email address of the individual to whom the invite was sent"),
    ]
    role: Annotated[Literal["owner", "reader"], Field(description="`owner` or `reader`")]
    status: Annotated[
        Literal["accepted", "expired", "pending"],
        Field(description="`accepted`,`expired`, or `pending`"),
    ]
    invited_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the invite was sent."),
    ]
    expires_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the invite expires."),
    ]
    accepted_at: Annotated[
        Optional[int],
        Field(description="The Unix timestamp (in seconds) of when the invite was accepted."),
    ] = None


class InviteListResponse(BaseModel):
    object: Annotated[Literal["list"], Field(description="The object type, which is always `list`")]
    data: List[Invite]
    first_id: Annotated[
        Optional[str],
        Field(description="The first `invite_id` in the retrieved `list`"),
    ] = None
    last_id: Annotated[
        Optional[str], Field(description="The last `invite_id` in the retrieved `list`")
    ] = None
    has_more: Annotated[
        Optional[bool],
        Field(
            description="The `has_more` property is used for pagination to indicate there are additional results."
        ),
    ] = None


class InviteRequest(BaseModel):
    email: Annotated[str, Field(description="Send an email to this address")]
    role: Annotated[Literal["reader", "owner"], Field(description="`owner` or `reader`")]


class InviteDeleteResponse(BaseModel):
    object: Annotated[
        Literal["organization.invite.deleted"],
        Field(description="The object type, which is always `organization.invite.deleted`"),
    ]
    id: str
    deleted: bool


class User(BaseModel):
    object: Annotated[
        Literal["organization.user"],
        Field(description="The object type, which is always `organization.user`"),
    ]
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    name: Annotated[str, Field(description="The name of the user")]
    email: Annotated[str, Field(description="The email address of the user")]
    role: Annotated[Literal["owner", "reader"], Field(description="`owner` or `reader`")]
    added_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the user was added."),
    ]


class UserListResponse(BaseModel):
    object: Literal["list"]
    data: List[User]
    first_id: str
    last_id: str
    has_more: bool


class UserRoleUpdateRequest(BaseModel):
    role: Annotated[Literal["owner", "reader"], Field(description="`owner` or `reader`")]


class UserDeleteResponse(BaseModel):
    object: Literal["organization.user.deleted"]
    id: str
    deleted: bool


class Project1(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    object: Annotated[
        Literal["organization.project"],
        Field(description="The object type, which is always `organization.project`"),
    ]
    name: Annotated[str, Field(description="The name of the project. This appears in reporting.")]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the project was created."),
    ]
    archived_at: Annotated[
        Optional[int],
        Field(
            description="The Unix timestamp (in seconds) of when the project was archived or `null`."
        ),
    ] = None
    status: Annotated[Literal["active", "archived"], Field(description="`active` or `archived`")]


class ProjectListResponse(BaseModel):
    object: Literal["list"]
    data: List[Project1]
    first_id: str
    last_id: str
    has_more: bool


class ProjectCreateRequest(BaseModel):
    name: Annotated[
        str,
        Field(description="The friendly name of the project, this name appears in reports."),
    ]


class ProjectUpdateRequest(BaseModel):
    name: Annotated[
        str,
        Field(description="The updated name of the project, this name appears in reports."),
    ]


class DefaultProjectErrorResponse(BaseModel):
    code: int
    message: str


class ProjectUser(BaseModel):
    object: Annotated[
        Literal["organization.project.user"],
        Field(description="The object type, which is always `organization.project.user`"),
    ]
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    name: Annotated[str, Field(description="The name of the user")]
    email: Annotated[str, Field(description="The email address of the user")]
    role: Annotated[Literal["owner", "member"], Field(description="`owner` or `member`")]
    added_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the project was added."),
    ]


class ProjectUserListResponse(BaseModel):
    object: str
    data: List[ProjectUser]
    first_id: str
    last_id: str
    has_more: bool


class ProjectUserCreateRequest(BaseModel):
    user_id: Annotated[str, Field(description="The ID of the user.")]
    role: Annotated[Literal["owner", "member"], Field(description="`owner` or `member`")]


class ProjectUserUpdateRequest(BaseModel):
    role: Annotated[Literal["owner", "member"], Field(description="`owner` or `member`")]


class ProjectUserDeleteResponse(BaseModel):
    object: Literal["organization.project.user.deleted"]
    id: str
    deleted: bool


class ProjectServiceAccount(BaseModel):
    object: Annotated[
        Literal["organization.project.service_account"],
        Field(
            description="The object type, which is always `organization.project.service_account`"
        ),
    ]
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    name: Annotated[str, Field(description="The name of the service account")]
    role: Annotated[Literal["owner", "member"], Field(description="`owner` or `member`")]
    created_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) of when the service account was created"
        ),
    ]


class ProjectServiceAccountListResponse(BaseModel):
    object: Literal["list"]
    data: List[ProjectServiceAccount]
    first_id: str
    last_id: str
    has_more: bool


class ProjectServiceAccountCreateRequest(BaseModel):
    name: Annotated[str, Field(description="The name of the service account being created.")]


class ProjectServiceAccountApiKey(BaseModel):
    object: Annotated[
        Literal["organization.project.service_account.api_key"],
        Field(
            description="The object type, which is always `organization.project.service_account.api_key`"
        ),
    ]
    value: str
    name: str
    created_at: int
    id: str


class ProjectServiceAccountDeleteResponse(BaseModel):
    object: Literal["organization.project.service_account.deleted"]
    id: str
    deleted: bool


class Owner(BaseModel):
    type: Annotated[
        Optional[Literal["user", "service_account"]],
        Field(description="`user` or `service_account`"),
    ] = None
    user: Optional[ProjectUser] = None
    service_account: Optional[ProjectServiceAccount] = None


class ProjectApiKey(BaseModel):
    object: Annotated[
        Literal["organization.project.api_key"],
        Field(description="The object type, which is always `organization.project.api_key`"),
    ]
    redacted_value: Annotated[str, Field(description="The redacted value of the API key")]
    name: Annotated[str, Field(description="The name of the API key")]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the API key was created"),
    ]
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints"),
    ]
    owner: Owner


class ProjectApiKeyListResponse(BaseModel):
    object: Literal["list"]
    data: List[ProjectApiKey]
    first_id: str
    last_id: str
    has_more: bool


class ProjectApiKeyDeleteResponse(BaseModel):
    object: Literal["organization.project.api_key.deleted"]
    id: str
    deleted: bool


class ListModelsResponse(BaseModel):
    object: Literal["list"]
    data: List[Model]


class CreateCompletionRequest(BaseModel):
    model: Annotated[
        Union[str, Literal["gpt-3.5-turbo-instruct", "davinci-002", "babbage-002"]],
        Field(
            description="ID of the model to use. You can use the [List models](/docs/api-reference/models/list) API to see all of your available models, or see our [Model overview](/docs/models/overview) for descriptions of them.\n"
        ),
    ]
    prompt: Annotated[
        Union[str, List[str], Prompt, Prompt1],
        Field(
            description="The prompt(s) to generate completions for, encoded as a string, array of strings, array of tokens, or array of token arrays.\n\nNote that <|endoftext|> is the document separator that the model sees during training, so if a prompt is not specified the model will generate as if from the beginning of a new document.\n"
        ),
    ]
    best_of: Annotated[
        Optional[int],
        Field(
            description='Generates `best_of` completions server-side and returns the "best" (the one with the highest log probability per token). Results cannot be streamed.\n\nWhen used with `n`, `best_of` controls the number of candidate completions and `n` specifies how many to return – `best_of` must be greater than `n`.\n\n**Note:** Because this parameter generates many completions, it can quickly consume your token quota. Use carefully and ensure that you have reasonable settings for `max_tokens` and `stop`.\n',
            ge=0,
            le=20,
        ),
    ] = 1
    echo: Annotated[
        Optional[bool],
        Field(description="Echo back the prompt in addition to the completion\n"),
    ] = False
    frequency_penalty: Annotated[
        Optional[float],
        Field(
            description="Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.\n\n[See more information about frequency and presence penalties.](/docs/guides/text-generation/parameter-details)\n",
            ge=-2.0,
            le=2.0,
        ),
    ] = 0
    logit_bias: Annotated[
        Optional[Dict[str, int]],
        Field(
            description='Modify the likelihood of specified tokens appearing in the completion.\n\nAccepts a JSON object that maps tokens (specified by their token ID in the GPT tokenizer) to an associated bias value from -100 to 100. You can use this [tokenizer tool](/tokenizer?view=bpe) to convert text to token IDs. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token.\n\nAs an example, you can pass `{"50256": -100}` to prevent the <|endoftext|> token from being generated.\n'
        ),
    ] = None
    logprobs: Annotated[
        Optional[int],
        Field(
            description="Include the log probabilities on the `logprobs` most likely output tokens, as well the chosen tokens. For example, if `logprobs` is 5, the API will return a list of the 5 most likely tokens. The API will always return the `logprob` of the sampled token, so there may be up to `logprobs+1` elements in the response.\n\nThe maximum value for `logprobs` is 5.\n",
            ge=0,
            le=5,
        ),
    ] = None
    max_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of [tokens](/tokenizer) that can be generated in the completion.\n\nThe token count of your prompt plus `max_tokens` cannot exceed the model's context length. [Example Python code](https://cookbook.openai.com/examples/how_to_count_tokens_with_tiktoken) for counting tokens.\n",
            example=16,
            ge=0,
        ),
    ] = 16
    n: Annotated[
        Optional[int],
        Field(
            description="How many completions to generate for each prompt.\n\n**Note:** Because this parameter generates many completions, it can quickly consume your token quota. Use carefully and ensure that you have reasonable settings for `max_tokens` and `stop`.\n",
            example=1,
            ge=1,
            le=128,
        ),
    ] = 1
    presence_penalty: Annotated[
        Optional[float],
        Field(
            description="Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the text so far, increasing the model's likelihood to talk about new topics.\n\n[See more information about frequency and presence penalties.](/docs/guides/text-generation/parameter-details)\n",
            ge=-2.0,
            le=2.0,
        ),
    ] = 0
    seed: Annotated[
        Optional[int],
        Field(
            description="If specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result.\n\nDeterminism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.\n",
            ge=-9223372036854775808,
            le=9223372036854775807,
        ),
    ] = None
    stop: Annotated[
        Optional[Union[str, Stop]],
        Field(
            description="Up to 4 sequences where the API will stop generating further tokens. The returned text will not contain the stop sequence.\n"
        ),
    ] = None
    stream: Annotated[
        Optional[bool],
        Field(
            description="Whether to stream back partial progress. If set, tokens will be sent as data-only [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format) as they become available, with the stream terminated by a `data: [DONE]` message. [Example Python code](https://cookbook.openai.com/examples/how_to_stream_completions).\n"
        ),
    ] = False
    stream_options: Optional[ChatCompletionStreamOptions] = None
    suffix: Annotated[
        Optional[str],
        Field(
            description="The suffix that comes after a completion of inserted text.\n\nThis parameter is only supported for `gpt-3.5-turbo-instruct`.\n",
            example="test.",
        ),
    ] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n\nWe generally recommend altering this or `top_p` but not both.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or `temperature` but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None


class CreateCompletionResponse(BaseModel):
    id: Annotated[str, Field(description="A unique identifier for the completion.")]
    choices: Annotated[
        List[Choice],
        Field(
            description="The list of completion choices the model generated for the input prompt."
        ),
    ]
    created: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) of when the completion was created."),
    ]
    model: Annotated[str, Field(description="The model used for completion.")]
    system_fingerprint: Annotated[
        Optional[str],
        Field(
            description="This fingerprint represents the backend configuration that the model runs with.\n\nCan be used in conjunction with the `seed` request parameter to understand when backend changes have been made that might impact determinism.\n"
        ),
    ] = None
    object: Annotated[
        Literal["text_completion"],
        Field(description='The object type, which is always "text_completion"'),
    ]
    usage: Optional[CompletionUsage] = None


class ChatCompletionTool(BaseModel):
    type: Annotated[
        Literal["function"],
        Field(description="The type of the tool. Currently, only `function` is supported."),
    ]
    function: FunctionObject


class ChatCompletionToolChoiceOption(BaseModel):
    __root__: Annotated[
        Union[Literal["none", "auto", "required"], ChatCompletionNamedToolChoice],
        Field(
            description='Controls which (if any) tool is called by the model.\n`none` means the model will not call any tool and instead generates a message.\n`auto` means the model can pick between generating a message or calling one or more tools.\n`required` means the model must call one or more tools.\nSpecifying a particular tool via `{"type": "function", "function": {"name": "my_function"}}` forces the model to call that tool.\n\n`none` is the default when no tools are present. `auto` is the default if tools are present.\n'
        ),
    ]


class ChatCompletionMessageToolCalls(BaseModel):
    __root__: Annotated[
        List[ChatCompletionMessageToolCall],
        Field(description="The tool calls generated by the model, such as function calls."),
    ]


class ChatCompletionResponseMessage(BaseModel):
    content: Annotated[str, Field(description="The contents of the message.")]
    refusal: Annotated[str, Field(description="The refusal message generated by the model.")]
    tool_calls: Optional[ChatCompletionMessageToolCalls] = None
    role: Annotated[
        Literal["assistant"],
        Field(description="The role of the author of this message."),
    ]
    function_call: Annotated[
        Optional[FunctionCall],
        Field(
            description="Deprecated and replaced by `tool_calls`. The name and arguments of a function that should be called, as generated by the model."
        ),
    ] = None


class Choice1(BaseModel):
    finish_reason: Annotated[
        Literal["stop", "length", "tool_calls", "content_filter", "function_call"],
        Field(
            description="The reason the model stopped generating tokens. This will be `stop` if the model hit a natural stop point or a provided stop sequence,\n`length` if the maximum number of tokens specified in the request was reached,\n`content_filter` if content was omitted due to a flag from our content filters,\n`tool_calls` if the model called a tool, or `function_call` (deprecated) if the model called a function.\n"
        ),
    ]
    index: Annotated[int, Field(description="The index of the choice in the list of choices.")]
    message: ChatCompletionResponseMessage
    logprobs: Annotated[Logprobs2, Field(description="Log probability information for the choice.")]


class CreateChatCompletionResponse(BaseModel):
    id: Annotated[str, Field(description="A unique identifier for the chat completion.")]
    choices: Annotated[
        List[Choice1],
        Field(
            description="A list of chat completion choices. Can be more than one if `n` is greater than 1."
        ),
    ]
    created: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) of when the chat completion was created."
        ),
    ]
    model: Annotated[str, Field(description="The model used for the chat completion.")]
    service_tier: Annotated[
        Optional[Literal["scale", "default"]],
        Field(
            description="The service tier used for processing the request. This field is only included if the `service_tier` parameter is specified in the request.",
            example="scale",
        ),
    ] = None
    system_fingerprint: Annotated[
        Optional[str],
        Field(
            description="This fingerprint represents the backend configuration that the model runs with.\n\nCan be used in conjunction with the `seed` request parameter to understand when backend changes have been made that might impact determinism.\n"
        ),
    ] = None
    object: Annotated[
        Literal["chat.completion"],
        Field(description="The object type, which is always `chat.completion`."),
    ]
    usage: Optional[CompletionUsage] = None


class Choice2(BaseModel):
    finish_reason: Annotated[
        Literal["stop", "length", "function_call", "content_filter"],
        Field(
            description="The reason the model stopped generating tokens. This will be `stop` if the model hit a natural stop point or a provided stop sequence, `length` if the maximum number of tokens specified in the request was reached, `content_filter` if content was omitted due to a flag from our content filters, or `function_call` if the model called a function.\n"
        ),
    ]
    index: Annotated[int, Field(description="The index of the choice in the list of choices.")]
    message: ChatCompletionResponseMessage


class CreateChatCompletionFunctionResponse(BaseModel):
    id: Annotated[str, Field(description="A unique identifier for the chat completion.")]
    choices: Annotated[
        List[Choice2],
        Field(
            description="A list of chat completion choices. Can be more than one if `n` is greater than 1."
        ),
    ]
    created: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) of when the chat completion was created."
        ),
    ]
    model: Annotated[str, Field(description="The model used for the chat completion.")]
    system_fingerprint: Annotated[
        Optional[str],
        Field(
            description="This fingerprint represents the backend configuration that the model runs with.\n\nCan be used in conjunction with the `seed` request parameter to understand when backend changes have been made that might impact determinism.\n"
        ),
    ] = None
    object: Annotated[
        Literal["chat.completion"],
        Field(description="The object type, which is always `chat.completion`."),
    ]
    usage: Optional[CompletionUsage] = None


class ImagesResponse(BaseModel):
    created: int
    data: List[Image]


class ListFilesResponse(BaseModel):
    data: List[OpenAIFile]
    object: Literal["list"]


class ListFineTuningJobEventsResponse(BaseModel):
    data: List[FineTuningJobEvent]
    object: Literal["list"]


class ListFineTuningJobCheckpointsResponse(BaseModel):
    data: List[FineTuningJobCheckpoint]
    object: Literal["list"]
    first_id: Optional[str] = None
    last_id: Optional[str] = None
    has_more: bool


class CreateEmbeddingResponse(BaseModel):
    data: Annotated[
        List[Embedding],
        Field(description="The list of embeddings generated by the model."),
    ]
    model: Annotated[
        str, Field(description="The name of the model used to generate the embedding.")
    ]
    object: Annotated[
        Literal["list"], Field(description='The object type, which is always "list".')
    ]
    usage: Annotated[Usage1, Field(description="The usage information for the request.")]


class FineTuningJob(BaseModel):
    id: Annotated[
        str,
        Field(description="The object identifier, which can be referenced in the API endpoints."),
    ]
    created_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the fine-tuning job was created."
        ),
    ]
    error: Annotated[
        Error1,
        Field(
            description="For fine-tuning jobs that have `failed`, this will contain more information on the cause of the failure."
        ),
    ]
    fine_tuned_model: Annotated[
        str,
        Field(
            description="The name of the fine-tuned model that is being created. The value will be null if the fine-tuning job is still running."
        ),
    ]
    finished_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the fine-tuning job was finished. The value will be null if the fine-tuning job is still running."
        ),
    ]
    hyperparameters: Annotated[
        Hyperparameters1,
        Field(
            description="The hyperparameters used for the fine-tuning job. See the [fine-tuning guide](/docs/guides/fine-tuning) for more details."
        ),
    ]
    model: Annotated[str, Field(description="The base model that is being fine-tuned.")]
    object: Annotated[
        Literal["fine_tuning.job"],
        Field(description='The object type, which is always "fine_tuning.job".'),
    ]
    organization_id: Annotated[
        str, Field(description="The organization that owns the fine-tuning job.")
    ]
    result_files: Annotated[
        List[str],
        Field(
            description="The compiled results file ID(s) for the fine-tuning job. You can retrieve the results with the [Files API](/docs/api-reference/files/retrieve-contents)."
        ),
    ]
    status: Annotated[
        Literal["validating_files", "queued", "running", "succeeded", "failed", "cancelled"],
        Field(
            description="The current status of the fine-tuning job, which can be either `validating_files`, `queued`, `running`, `succeeded`, `failed`, or `cancelled`."
        ),
    ]
    trained_tokens: Annotated[
        int,
        Field(
            description="The total number of billable tokens processed by this fine-tuning job. The value will be null if the fine-tuning job is still running."
        ),
    ]
    training_file: Annotated[
        str,
        Field(
            description="The file ID used for training. You can retrieve the training data with the [Files API](/docs/api-reference/files/retrieve-contents)."
        ),
    ]
    validation_file: Annotated[
        str,
        Field(
            description="The file ID used for validation. You can retrieve the validation results with the [Files API](/docs/api-reference/files/retrieve-contents)."
        ),
    ]
    integrations: Annotated[
        Optional[List[FineTuningIntegration]],
        Field(
            description="A list of integrations to enable for this fine-tuning job.",
            max_items=5,
        ),
    ] = None
    seed: Annotated[int, Field(description="The seed used for the fine-tuning job.")]
    estimated_finish: Annotated[
        Optional[int],
        Field(
            description="The Unix timestamp (in seconds) for when the fine-tuning job is estimated to finish. The value will be null if the fine-tuning job is not running."
        ),
    ] = None


class AssistantObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["assistant"],
        Field(description="The object type, which is always `assistant`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the assistant was created."),
    ]
    name: Annotated[
        str,
        Field(
            description="The name of the assistant. The maximum length is 256 characters.\n",
            max_length=256,
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="The description of the assistant. The maximum length is 512 characters.\n",
            max_length=512,
        ),
    ]
    model: Annotated[
        str,
        Field(
            description="ID of the model to use. You can use the [List models](/docs/api-reference/models/list) API to see all of your available models, or see our [Model overview](/docs/models/overview) for descriptions of them.\n"
        ),
    ]
    instructions: Annotated[
        str,
        Field(
            description="The system instructions that the assistant uses. The maximum length is 256,000 characters.\n",
            max_length=256000,
        ),
    ]
    tools: Annotated[
        List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]],
        Field(
            description="A list of tool enabled on the assistant. There can be a maximum of 128 tools per assistant. Tools can be of types `code_interpreter`, `file_search`, or `function`.\n",
            max_items=128,
        ),
    ]
    tool_resources: Annotated[
        Optional[ToolResources],
        Field(
            description="A set of resources that are used by the assistant's tools. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or temperature but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    response_format: Optional[AssistantsApiResponseFormatOption] = None


class CreateAssistantRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    model: Annotated[
        Union[
            str,
            Literal[
                "gpt-4o",
                "gpt-4o-2024-08-06",
                "gpt-4o-2024-05-13",
                "gpt-4o-2024-08-06",
                "gpt-4o-mini",
                "gpt-4o-mini-2024-07-18",
                "gpt-4-turbo",
                "gpt-4-turbo-2024-04-09",
                "gpt-4-0125-preview",
                "gpt-4-turbo-preview",
                "gpt-4-1106-preview",
                "gpt-4-vision-preview",
                "gpt-4",
                "gpt-4-0314",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0314",
                "gpt-4-32k-0613",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-1106",
                "gpt-3.5-turbo-0125",
                "gpt-3.5-turbo-16k-0613",
            ],
        ],
        Field(
            description="ID of the model to use. You can use the [List models](/docs/api-reference/models/list) API to see all of your available models, or see our [Model overview](/docs/models/overview) for descriptions of them.\n",
            example="gpt-4o",
        ),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="The name of the assistant. The maximum length is 256 characters.\n",
            max_length=256,
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="The description of the assistant. The maximum length is 512 characters.\n",
            max_length=512,
        ),
    ] = None
    instructions: Annotated[
        Optional[str],
        Field(
            description="The system instructions that the assistant uses. The maximum length is 256,000 characters.\n",
            max_length=256000,
        ),
    ] = None
    tools: Annotated[
        Optional[List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]]],
        Field(
            description="A list of tool enabled on the assistant. There can be a maximum of 128 tools per assistant. Tools can be of types `code_interpreter`, `file_search`, or `function`.\n",
            max_items=128,
        ),
    ] = []
    tool_resources: Annotated[
        Optional[ToolResources1],
        Field(
            description="A set of resources that are used by the assistant's tools. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or temperature but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    response_format: Optional[AssistantsApiResponseFormatOption] = None


class ModifyAssistantRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    model: Annotated[
        Optional[str],
        Field(
            description="ID of the model to use. You can use the [List models](/docs/api-reference/models/list) API to see all of your available models, or see our [Model overview](/docs/models/overview) for descriptions of them.\n"
        ),
    ] = None
    name: Annotated[
        Optional[str],
        Field(
            description="The name of the assistant. The maximum length is 256 characters.\n",
            max_length=256,
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="The description of the assistant. The maximum length is 512 characters.\n",
            max_length=512,
        ),
    ] = None
    instructions: Annotated[
        Optional[str],
        Field(
            description="The system instructions that the assistant uses. The maximum length is 256,000 characters.\n",
            max_length=256000,
        ),
    ] = None
    tools: Annotated[
        Optional[List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]]],
        Field(
            description="A list of tool enabled on the assistant. There can be a maximum of 128 tools per assistant. Tools can be of types `code_interpreter`, `file_search`, or `function`.\n",
            max_items=128,
        ),
    ] = []
    tool_resources: Annotated[
        Optional[ToolResources2],
        Field(
            description="A set of resources that are used by the assistant's tools. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or temperature but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    response_format: Optional[AssistantsApiResponseFormatOption] = None


class ListAssistantsResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[AssistantObject]
    first_id: Annotated[str, Field(example="asst_abc123")]
    last_id: Annotated[str, Field(example="asst_abc456")]
    has_more: Annotated[bool, Field(example=False)]


class AssistantsApiToolChoiceOption(BaseModel):
    __root__: Annotated[
        Union[Literal["none", "auto", "required"], AssistantsNamedToolChoice],
        Field(
            description='Controls which (if any) tool is called by the model.\n`none` means the model will not call any tools and instead generates a message.\n`auto` is the default value and means the model can pick between generating a message or calling one or more tools.\n`required` means the model must call one or more tools before responding to the user.\nSpecifying a particular tool like `{"type": "file_search"}` or `{"type": "function", "function": {"name": "my_function"}}` forces the model to call that tool.\n'
        ),
    ]


class SubmitToolOutputs(BaseModel):
    tool_calls: Annotated[
        List[RunToolCallObject], Field(description="A list of the relevant tool calls.")
    ]


class RequiredAction(BaseModel):
    type: Annotated[
        Literal["submit_tool_outputs"],
        Field(description="For now, this is always `submit_tool_outputs`."),
    ]
    submit_tool_outputs: Annotated[
        SubmitToolOutputs,
        Field(description="Details on the tool outputs needed for this run to continue."),
    ]


class RunObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["thread.run"],
        Field(description="The object type, which is always `thread.run`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run was created."),
    ]
    thread_id: Annotated[
        str,
        Field(
            description="The ID of the [thread](/docs/api-reference/threads) that was executed on as a part of this run."
        ),
    ]
    assistant_id: Annotated[
        str,
        Field(
            description="The ID of the [assistant](/docs/api-reference/assistants) used for execution of this run."
        ),
    ]
    status: Annotated[
        Literal[
            "queued",
            "in_progress",
            "requires_action",
            "cancelling",
            "cancelled",
            "failed",
            "completed",
            "incomplete",
            "expired",
        ],
        Field(
            description="The status of the run, which can be either `queued`, `in_progress`, `requires_action`, `cancelling`, `cancelled`, `failed`, `completed`, `incomplete`, or `expired`."
        ),
    ]
    required_action: Annotated[
        RequiredAction,
        Field(
            description="Details on the action required to continue the run. Will be `null` if no action is required."
        ),
    ]
    last_error: Annotated[
        LastError,
        Field(
            description="The last error associated with this run. Will be `null` if there are no errors."
        ),
    ]
    expires_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run will expire."),
    ]
    started_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run was started."),
    ]
    cancelled_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run was cancelled."),
    ]
    failed_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run failed."),
    ]
    completed_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run was completed."),
    ]
    incomplete_details: Annotated[
        IncompleteDetails,
        Field(
            description="Details on why the run is incomplete. Will be `null` if the run is not incomplete."
        ),
    ]
    model: Annotated[
        str,
        Field(
            description="The model that the [assistant](/docs/api-reference/assistants) used for this run."
        ),
    ]
    instructions: Annotated[
        str,
        Field(
            description="The instructions that the [assistant](/docs/api-reference/assistants) used for this run."
        ),
    ]
    tools: Annotated[
        List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]],
        Field(
            description="The list of tools that the [assistant](/docs/api-reference/assistants) used for this run.",
            max_items=20,
        ),
    ]
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]
    usage: RunCompletionUsage
    temperature: Annotated[
        Optional[float],
        Field(description="The sampling temperature used for this run. If not set, defaults to 1."),
    ] = None
    top_p: Annotated[
        Optional[float],
        Field(
            description="The nucleus sampling value used for this run. If not set, defaults to 1."
        ),
    ] = None
    max_prompt_tokens: Annotated[
        int,
        Field(
            description="The maximum number of prompt tokens specified to have been used over the course of the run.\n",
            ge=256,
        ),
    ]
    max_completion_tokens: Annotated[
        int,
        Field(
            description="The maximum number of completion tokens specified to have been used over the course of the run.\n",
            ge=256,
        ),
    ]
    truncation_strategy: TruncationObject
    tool_choice: AssistantsApiToolChoiceOption
    parallel_tool_calls: ParallelToolCalls
    response_format: AssistantsApiResponseFormatOption


class ListRunsResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[RunObject]
    first_id: Annotated[str, Field(example="run_abc123")]
    last_id: Annotated[str, Field(example="run_abc456")]
    has_more: Annotated[bool, Field(example=False)]


class Content4(BaseModel):
    __root__: Annotated[
        List[
            Union[
                MessageContentImageFileObject,
                MessageContentImageUrlObject,
                MessageRequestContentTextObject,
            ]
        ],
        Field(
            description="An array of content parts with a defined type, each can be of type `text` or images can be passed with `image_url` or `image_file`. Image types are only supported on [Vision-compatible models](/docs/models/overview).",
            min_items=1,
            title="Array of content parts",
        ),
    ]


class CreateMessageRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    role: Annotated[
        Literal["user", "assistant"],
        Field(
            description="The role of the entity that is creating the message. Allowed values include:\n- `user`: Indicates the message is sent by an actual user and should be used in most cases to represent user-generated messages.\n- `assistant`: Indicates the message is generated by the assistant. Use this value to insert messages from the assistant into the conversation.\n"
        ),
    ]
    content: Union[str, Content4]
    attachments: Annotated[
        Optional[List[Attachment]],
        Field(
            description="A list of files attached to the message, and the tools they should be added to."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class Text(BaseModel):
    value: Annotated[str, Field(description="The data that makes up the text.")]
    annotations: List[
        Union[
            MessageContentTextAnnotationsFileCitationObject,
            MessageContentTextAnnotationsFilePathObject,
        ]
    ]


class MessageContentTextObject(BaseModel):
    type: Annotated[Literal["text"], Field(description="Always `text`.")]
    text: Text


class Text1(BaseModel):
    value: Annotated[Optional[str], Field(description="The data that makes up the text.")] = None
    annotations: Optional[
        List[
            Union[
                MessageDeltaContentTextAnnotationsFileCitationObject,
                MessageDeltaContentTextAnnotationsFilePathObject,
            ]
        ]
    ] = None


class MessageDeltaContentTextObject(BaseModel):
    index: Annotated[int, Field(description="The index of the content part in the message.")]
    type: Annotated[Literal["text"], Field(description="Always `text`.")]
    text: Optional[Text1] = None


class CodeInterpreter7(BaseModel):
    input: Annotated[str, Field(description="The input to the Code Interpreter tool call.")]
    outputs: Annotated[
        List[
            Union[
                RunStepDetailsToolCallsCodeOutputLogsObject,
                RunStepDetailsToolCallsCodeOutputImageObject,
            ]
        ],
        Field(
            description="The outputs from the Code Interpreter tool call. Code Interpreter can output one or more items, including text (`logs`) or images (`image`). Each of these are represented by a different object type."
        ),
    ]


class RunStepDetailsToolCallsCodeObject(BaseModel):
    id: Annotated[str, Field(description="The ID of the tool call.")]
    type: Annotated[
        Literal["code_interpreter"],
        Field(
            description="The type of tool call. This is always going to be `code_interpreter` for this type of tool call."
        ),
    ]
    code_interpreter: Annotated[
        CodeInterpreter7,
        Field(description="The Code Interpreter tool call definition."),
    ]


class CodeInterpreter8(BaseModel):
    input: Annotated[
        Optional[str], Field(description="The input to the Code Interpreter tool call.")
    ] = None
    outputs: Annotated[
        Optional[
            List[
                Union[
                    RunStepDeltaStepDetailsToolCallsCodeOutputLogsObject,
                    RunStepDeltaStepDetailsToolCallsCodeOutputImageObject,
                ]
            ]
        ],
        Field(
            description="The outputs from the Code Interpreter tool call. Code Interpreter can output one or more items, including text (`logs`) or images (`image`). Each of these are represented by a different object type."
        ),
    ] = None


class RunStepDeltaStepDetailsToolCallsCodeObject(BaseModel):
    index: Annotated[int, Field(description="The index of the tool call in the tool calls array.")]
    id: Annotated[Optional[str], Field(description="The ID of the tool call.")] = None
    type: Annotated[
        Literal["code_interpreter"],
        Field(
            description="The type of tool call. This is always going to be `code_interpreter` for this type of tool call."
        ),
    ]
    code_interpreter: Annotated[
        Optional[CodeInterpreter8],
        Field(description="The Code Interpreter tool call definition."),
    ] = None


class CreateVectorStoreRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    file_ids: Annotated[
        Optional[List[str]],
        Field(
            description="A list of [File](/docs/api-reference/files) IDs that the vector store should use. Useful for tools like `file_search` that can access files.",
            max_items=500,
        ),
    ] = None
    name: Annotated[Optional[str], Field(description="The name of the vector store.")] = None
    expires_after: Optional[VectorStoreExpirationAfter] = None
    chunking_strategy: Annotated[
        Optional[Union[AutoChunkingStrategyRequestParam, StaticChunkingStrategyRequestParam]],
        Field(
            description="The chunking strategy used to chunk the file(s). If not set, will use the `auto` strategy. Only applicable if `file_ids` is non-empty."
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class StaticChunkingStrategyResponseParam(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[Literal["static"], Field(description="Always `static`.")]
    static: StaticChunkingStrategy


class RunStreamEvent1(BaseModel):
    event: Literal["thread.run.created"]
    data: RunObject


class RunStreamEvent2(BaseModel):
    event: Literal["thread.run.queued"]
    data: RunObject


class RunStreamEvent3(BaseModel):
    event: Literal["thread.run.in_progress"]
    data: RunObject


class RunStreamEvent4(BaseModel):
    event: Literal["thread.run.requires_action"]
    data: RunObject


class RunStreamEvent5(BaseModel):
    event: Literal["thread.run.completed"]
    data: RunObject


class RunStreamEvent6(BaseModel):
    event: Literal["thread.run.incomplete"]
    data: RunObject


class RunStreamEvent7(BaseModel):
    event: Literal["thread.run.failed"]
    data: RunObject


class RunStreamEvent8(BaseModel):
    event: Literal["thread.run.cancelling"]
    data: RunObject


class RunStreamEvent9(BaseModel):
    event: Literal["thread.run.cancelled"]
    data: RunObject


class RunStreamEvent10(BaseModel):
    event: Literal["thread.run.expired"]
    data: RunObject


class RunStreamEvent(BaseModel):
    __root__: Union[
        RunStreamEvent1,
        RunStreamEvent2,
        RunStreamEvent3,
        RunStreamEvent4,
        RunStreamEvent5,
        RunStreamEvent6,
        RunStreamEvent7,
        RunStreamEvent8,
        RunStreamEvent9,
        RunStreamEvent10,
    ]


class ProjectServiceAccountCreateResponse(BaseModel):
    object: Literal["organization.project.service_account"]
    id: str
    name: str
    role: Annotated[
        Literal["member"],
        Field(description="Service accounts can only have one role of type `member`"),
    ]
    created_at: int
    api_key: ProjectServiceAccountApiKey


class ChatCompletionRequestAssistantMessage(BaseModel):
    content: Annotated[
        Optional[Union[str, Content2]],
        Field(
            description="The contents of the assistant message. Required unless `tool_calls` or `function_call` is specified.\n"
        ),
    ] = None
    refusal: Annotated[
        Optional[str], Field(description="The refusal message by the assistant.")
    ] = None
    role: Annotated[
        Literal["assistant"],
        Field(description="The role of the messages author, in this case `assistant`."),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="An optional name for the participant. Provides the model information to differentiate between participants of the same role."
        ),
    ] = None
    tool_calls: Optional[ChatCompletionMessageToolCalls] = None
    function_call: Annotated[
        Optional[FunctionCall],
        Field(
            description="Deprecated and replaced by `tool_calls`. The name and arguments of a function that should be called, as generated by the model."
        ),
    ] = None


class FineTuneChatCompletionRequestAssistantMessage(ChatCompletionRequestAssistantMessage):
    weight: Annotated[
        Optional[Literal[0, 1]],
        Field(description="Controls whether the assistant message is trained against (0 or 1)"),
    ] = None
    role: Annotated[
        Literal["assistant"],
        Field(description="The role of the messages author, in this case `assistant`."),
    ]


class ListPaginatedFineTuningJobsResponse(BaseModel):
    data: List[FineTuningJob]
    has_more: bool
    object: Literal["list"]


class FinetuneChatRequestInput(BaseModel):
    messages: Annotated[
        Optional[
            List[
                Union[
                    ChatCompletionRequestSystemMessage,
                    ChatCompletionRequestUserMessage,
                    FineTuneChatCompletionRequestAssistantMessage,
                    ChatCompletionRequestToolMessage,
                    ChatCompletionRequestFunctionMessage,
                ]
            ]
        ],
        Field(min_items=1),
    ] = None
    tools: Annotated[
        Optional[List[ChatCompletionTool]],
        Field(description="A list of tools the model may generate JSON inputs for."),
    ] = None
    parallel_tool_calls: Optional[ParallelToolCalls] = None
    functions: Annotated[
        Optional[List[ChatCompletionFunctions]],
        Field(
            description="A list of functions the model may generate JSON inputs for.",
            max_items=128,
            min_items=1,
        ),
    ] = None


class CreateRunRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    assistant_id: Annotated[
        str,
        Field(
            description="The ID of the [assistant](/docs/api-reference/assistants) to use to execute this run."
        ),
    ]
    model: Annotated[
        Optional[
            Union[
                str,
                Literal[
                    "gpt-4o",
                    "gpt-4o-2024-08-06",
                    "gpt-4o-2024-05-13",
                    "gpt-4o-2024-08-06",
                    "gpt-4o-mini",
                    "gpt-4o-mini-2024-07-18",
                    "gpt-4-turbo",
                    "gpt-4-turbo-2024-04-09",
                    "gpt-4-0125-preview",
                    "gpt-4-turbo-preview",
                    "gpt-4-1106-preview",
                    "gpt-4-vision-preview",
                    "gpt-4",
                    "gpt-4-0314",
                    "gpt-4-0613",
                    "gpt-4-32k",
                    "gpt-4-32k-0314",
                    "gpt-4-32k-0613",
                    "gpt-3.5-turbo",
                    "gpt-3.5-turbo-16k",
                    "gpt-3.5-turbo-0613",
                    "gpt-3.5-turbo-1106",
                    "gpt-3.5-turbo-0125",
                    "gpt-3.5-turbo-16k-0613",
                ],
            ]
        ],
        Field(
            description="The ID of the [Model](/docs/api-reference/models) to be used to execute this run. If a value is provided here, it will override the model associated with the assistant. If not, the model associated with the assistant will be used.",
            example="gpt-4o",
        ),
    ] = None
    instructions: Annotated[
        Optional[str],
        Field(
            description="Overrides the [instructions](/docs/api-reference/assistants/createAssistant) of the assistant. This is useful for modifying the behavior on a per-run basis."
        ),
    ] = None
    additional_instructions: Annotated[
        Optional[str],
        Field(
            description="Appends additional instructions at the end of the instructions for the run. This is useful for modifying the behavior on a per-run basis without overriding other instructions."
        ),
    ] = None
    additional_messages: Annotated[
        Optional[List[CreateMessageRequest]],
        Field(description="Adds additional messages to the thread before creating the run."),
    ] = None
    tools: Annotated[
        Optional[List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]]],
        Field(
            description="Override the tools the assistant can use for this run. This is useful for modifying the behavior on a per-run basis.",
            max_items=20,
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or temperature but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    stream: Annotated[
        Optional[bool],
        Field(
            description="If `true`, returns a stream of events that happen during the Run as server-sent events, terminating when the Run enters a terminal state with a `data: [DONE]` message.\n"
        ),
    ] = None
    max_prompt_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of prompt tokens that may be used over the course of the run. The run will make a best effort to use only the number of prompt tokens specified, across multiple turns of the run. If the run exceeds the number of prompt tokens specified, the run will end with status `incomplete`. See `incomplete_details` for more info.\n",
            ge=256,
        ),
    ] = None
    max_completion_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of completion tokens that may be used over the course of the run. The run will make a best effort to use only the number of completion tokens specified, across multiple turns of the run. If the run exceeds the number of completion tokens specified, the run will end with status `incomplete`. See `incomplete_details` for more info.\n",
            ge=256,
        ),
    ] = None
    truncation_strategy: Optional[TruncationObject] = None
    tool_choice: Optional[AssistantsApiToolChoiceOption] = None
    parallel_tool_calls: Optional[ParallelToolCalls] = None
    response_format: Optional[AssistantsApiResponseFormatOption] = None


class CreateThreadRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    messages: Annotated[
        Optional[List[CreateMessageRequest]],
        Field(
            description="A list of [messages](/docs/api-reference/messages) to start the thread with."
        ),
    ] = None
    tool_resources: Annotated[
        Optional[ToolResources5],
        Field(
            description="A set of resources that are made available to the assistant's tools in this thread. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None


class MessageObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["thread.message"],
        Field(description="The object type, which is always `thread.message`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the message was created."),
    ]
    thread_id: Annotated[
        str,
        Field(
            description="The [thread](/docs/api-reference/threads) ID that this message belongs to."
        ),
    ]
    status: Annotated[
        Literal["in_progress", "incomplete", "completed"],
        Field(
            description="The status of the message, which can be either `in_progress`, `incomplete`, or `completed`."
        ),
    ]
    incomplete_details: Annotated[
        IncompleteDetails1,
        Field(description="On an incomplete message, details about why the message is incomplete."),
    ]
    completed_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the message was completed."),
    ]
    incomplete_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the message was marked as incomplete."
        ),
    ]
    role: Annotated[
        Literal["user", "assistant"],
        Field(description="The entity that produced the message. One of `user` or `assistant`."),
    ]
    content: Annotated[
        List[
            Union[
                MessageContentImageFileObject,
                MessageContentImageUrlObject,
                MessageContentTextObject,
                MessageContentRefusalObject,
            ]
        ],
        Field(description="The content of the message in array of text and/or images."),
    ]
    assistant_id: Annotated[
        str,
        Field(
            description="If applicable, the ID of the [assistant](/docs/api-reference/assistants) that authored this message."
        ),
    ]
    run_id: Annotated[
        str,
        Field(
            description="The ID of the [run](/docs/api-reference/runs) associated with the creation of this message. Value is `null` when messages are created manually using the create message or create thread endpoints."
        ),
    ]
    attachments: Annotated[
        List[Attachment],
        Field(
            description="A list of files attached to the message, and the tools they were added to."
        ),
    ]
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]


class Delta(BaseModel):
    role: Annotated[
        Optional[Literal["user", "assistant"]],
        Field(description="The entity that produced the message. One of `user` or `assistant`."),
    ] = None
    content: Annotated[
        Optional[
            List[
                Union[
                    MessageDeltaContentImageFileObject,
                    MessageDeltaContentTextObject,
                    MessageDeltaContentRefusalObject,
                    MessageDeltaContentImageUrlObject,
                ]
            ]
        ],
        Field(description="The content of the message in array of text and/or images."),
    ] = None


class MessageDeltaObject(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The identifier of the message, which can be referenced in API endpoints."
        ),
    ]
    object: Annotated[
        Literal["thread.message.delta"],
        Field(description="The object type, which is always `thread.message.delta`."),
    ]
    delta: Annotated[
        Delta,
        Field(description="The delta containing the fields that have changed on the Message."),
    ]


class ListMessagesResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[MessageObject]
    first_id: Annotated[str, Field(example="msg_abc123")]
    last_id: Annotated[str, Field(example="msg_abc123")]
    has_more: Annotated[bool, Field(example=False)]


class RunStepDetailsToolCallsObject(BaseModel):
    type: Annotated[Literal["tool_calls"], Field(description="Always `tool_calls`.")]
    tool_calls: Annotated[
        List[
            Union[
                RunStepDetailsToolCallsCodeObject,
                RunStepDetailsToolCallsFileSearchObject,
                RunStepDetailsToolCallsFunctionObject,
            ]
        ],
        Field(
            description="An array of tool calls the run step was involved in. These can be associated with one of three types of tools: `code_interpreter`, `file_search`, or `function`.\n"
        ),
    ]


class RunStepDeltaStepDetailsToolCallsObject(BaseModel):
    type: Annotated[Literal["tool_calls"], Field(description="Always `tool_calls`.")]
    tool_calls: Annotated[
        Optional[
            List[
                Union[
                    RunStepDeltaStepDetailsToolCallsCodeObject,
                    RunStepDeltaStepDetailsToolCallsFileSearchObject,
                    RunStepDeltaStepDetailsToolCallsFunctionObject,
                ]
            ]
        ],
        Field(
            description="An array of tool calls the run step was involved in. These can be associated with one of three types of tools: `code_interpreter`, `file_search`, or `function`.\n"
        ),
    ] = None


class VectorStoreFileObject(BaseModel):
    id: Annotated[
        str,
        Field(description="The identifier, which can be referenced in API endpoints."),
    ]
    object: Annotated[
        Literal["vector_store.file"],
        Field(description="The object type, which is always `vector_store.file`."),
    ]
    usage_bytes: Annotated[
        int,
        Field(
            description="The total vector store usage in bytes. Note that this may be different from the original file size."
        ),
    ]
    created_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the vector store file was created."
        ),
    ]
    vector_store_id: Annotated[
        str,
        Field(
            description="The ID of the [vector store](/docs/api-reference/vector-stores/object) that the [File](/docs/api-reference/files) is attached to."
        ),
    ]
    status: Annotated[
        Literal["in_progress", "completed", "cancelled", "failed"],
        Field(
            description="The status of the vector store file, which can be either `in_progress`, `completed`, `cancelled`, or `failed`. The status `completed` indicates that the vector store file is ready for use."
        ),
    ]
    last_error: Annotated[
        LastError2,
        Field(
            description="The last error associated with this vector store file. Will be `null` if there are no errors."
        ),
    ]
    chunking_strategy: Annotated[
        Optional[Union[StaticChunkingStrategyResponseParam, OtherChunkingStrategyResponseParam]],
        Field(description="The strategy used to chunk the file."),
    ] = None


class ListVectorStoreFilesResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[VectorStoreFileObject]
    first_id: Annotated[str, Field(example="file-abc123")]
    last_id: Annotated[str, Field(example="file-abc456")]
    has_more: Annotated[bool, Field(example=False)]


class MessageStreamEvent1(BaseModel):
    event: Literal["thread.message.created"]
    data: MessageObject


class MessageStreamEvent2(BaseModel):
    event: Literal["thread.message.in_progress"]
    data: MessageObject


class MessageStreamEvent3(BaseModel):
    event: Literal["thread.message.delta"]
    data: MessageDeltaObject


class MessageStreamEvent4(BaseModel):
    event: Literal["thread.message.completed"]
    data: MessageObject


class MessageStreamEvent5(BaseModel):
    event: Literal["thread.message.incomplete"]
    data: MessageObject


class MessageStreamEvent(BaseModel):
    __root__: Union[
        MessageStreamEvent1,
        MessageStreamEvent2,
        MessageStreamEvent3,
        MessageStreamEvent4,
        MessageStreamEvent5,
    ]


class ChatCompletionRequestMessage(BaseModel):
    __root__: Annotated[
        Union[
            ChatCompletionRequestSystemMessage,
            ChatCompletionRequestUserMessage,
            ChatCompletionRequestAssistantMessage,
            ChatCompletionRequestToolMessage,
            ChatCompletionRequestFunctionMessage,
        ],
        Field(discriminator="role"),
    ]


class CreateChatCompletionRequest(BaseModel):
    messages: Annotated[
        List[ChatCompletionRequestMessage],
        Field(
            description="A list of messages comprising the conversation so far. [Example Python code](https://cookbook.openai.com/examples/how_to_format_inputs_to_chatgpt_models).",
            min_items=1,
        ),
    ]
    model: Annotated[
        Union[
            str,
            Literal[
                "gpt-4o",
                "gpt-4o-2024-05-13",
                "gpt-4o-2024-08-06",
                "chatgpt-4o-latest",
                "gpt-4o-mini",
                "gpt-4o-mini-2024-07-18",
                "gpt-4-turbo",
                "gpt-4-turbo-2024-04-09",
                "gpt-4-0125-preview",
                "gpt-4-turbo-preview",
                "gpt-4-1106-preview",
                "gpt-4-vision-preview",
                "gpt-4",
                "gpt-4-0314",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0314",
                "gpt-4-32k-0613",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-0301",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-1106",
                "gpt-3.5-turbo-0125",
                "gpt-3.5-turbo-16k-0613",
            ],
        ],
        Field(
            description="ID of the model to use. See the [model endpoint compatibility](/docs/models/model-endpoint-compatibility) table for details on which models work with the Chat API.",
            example="gpt-4o",
        ),
    ]
    frequency_penalty: Annotated[
        Optional[float],
        Field(
            description="Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.\n\n[See more information about frequency and presence penalties.](/docs/guides/text-generation/parameter-details)\n",
            ge=-2.0,
            le=2.0,
        ),
    ] = 0
    logit_bias: Annotated[
        Optional[Dict[str, int]],
        Field(
            description="Modify the likelihood of specified tokens appearing in the completion.\n\nAccepts a JSON object that maps tokens (specified by their token ID in the tokenizer) to an associated bias value from -100 to 100. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token.\n"
        ),
    ] = None
    logprobs: Annotated[
        Optional[bool],
        Field(
            description="Whether to return log probabilities of the output tokens or not. If true, returns the log probabilities of each output token returned in the `content` of `message`."
        ),
    ] = False
    top_logprobs: Annotated[
        Optional[int],
        Field(
            description="An integer between 0 and 20 specifying the number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to `true` if this parameter is used.",
            ge=0,
            le=20,
        ),
    ] = None
    max_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of [tokens](/tokenizer) that can be generated in the chat completion.\n\nThe total length of input tokens and generated tokens is limited by the model's context length. [Example Python code](https://cookbook.openai.com/examples/how_to_count_tokens_with_tiktoken) for counting tokens.\n"
        ),
    ] = None
    n: Annotated[
        Optional[int],
        Field(
            description="How many chat completion choices to generate for each input message. Note that you will be charged based on the number of generated tokens across all of the choices. Keep `n` as `1` to minimize costs.",
            example=1,
            ge=1,
            le=128,
        ),
    ] = 1
    presence_penalty: Annotated[
        Optional[float],
        Field(
            description="Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the text so far, increasing the model's likelihood to talk about new topics.\n\n[See more information about frequency and presence penalties.](/docs/guides/text-generation/parameter-details)\n",
            ge=-2.0,
            le=2.0,
        ),
    ] = 0
    response_format: Annotated[
        Optional[Union[ResponseFormatText, ResponseFormatJsonObject, ResponseFormatJsonSchema]],
        Field(
            description='An object specifying the format that the model must output. Compatible with [GPT-4o](/docs/models/gpt-4o), [GPT-4o mini](/docs/models/gpt-4o-mini), [GPT-4 Turbo](/docs/models/gpt-4-and-gpt-4-turbo) and all GPT-3.5 Turbo models newer than `gpt-3.5-turbo-1106`.\n\nSetting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured Outputs which guarantees the model will match your supplied JSON schema. Learn more in the [Structured Outputs guide](/docs/guides/structured-outputs).\n\nSetting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.\n\n**Important:** when using JSON mode, you **must** also instruct the model to produce JSON yourself via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request. Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length.\n'
        ),
    ] = None
    seed: Annotated[
        Optional[int],
        Field(
            description="This feature is in Beta.\nIf specified, our system will make a best effort to sample deterministically, such that repeated requests with the same `seed` and parameters should return the same result.\nDeterminism is not guaranteed, and you should refer to the `system_fingerprint` response parameter to monitor changes in the backend.\n",
            ge=-9223372036854775808,
            le=9223372036854775807,
        ),
    ] = None
    service_tier: Annotated[
        Optional[Literal["auto", "default"]],
        Field(
            description="Specifies the latency tier to use for processing the request. This parameter is relevant for customers subscribed to the scale tier service:\n  - If set to 'auto', the system will utilize scale tier credits until they are exhausted.\n  - If set to 'default', the request will be processed using the default service tier with a lower uptime SLA and no latency guarentee.\n  - When not set, the default behavior is 'auto'.\n\n  When this parameter is set, the response body will include the `service_tier` utilized.\n"
        ),
    ] = None
    stop: Annotated[
        Optional[Union[str, Stop1]],
        Field(description="Up to 4 sequences where the API will stop generating further tokens.\n"),
    ] = None
    stream: Annotated[
        Optional[bool],
        Field(
            description="If set, partial message deltas will be sent, like in ChatGPT. Tokens will be sent as data-only [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format) as they become available, with the stream terminated by a `data: [DONE]` message. [Example Python code](https://cookbook.openai.com/examples/how_to_stream_completions).\n"
        ),
    ] = False
    stream_options: Optional[ChatCompletionStreamOptions] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n\nWe generally recommend altering this or `top_p` but not both.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or `temperature` but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    tools: Annotated[
        Optional[List[ChatCompletionTool]],
        Field(
            description="A list of tools the model may call. Currently, only functions are supported as a tool. Use this to provide a list of functions the model may generate JSON inputs for. A max of 128 functions are supported.\n"
        ),
    ] = None
    tool_choice: Optional[ChatCompletionToolChoiceOption] = None
    parallel_tool_calls: Optional[ParallelToolCalls] = None
    user: Annotated[
        Optional[str],
        Field(
            description="A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. [Learn more](/docs/guides/safety-best-practices/end-user-ids).\n",
            example="user-1234",
        ),
    ] = None
    function_call: Annotated[
        Optional[Union[Literal["none", "auto"], ChatCompletionFunctionCallOption]],
        Field(
            description='Deprecated in favor of `tool_choice`.\n\nControls which (if any) function is called by the model.\n`none` means the model will not call a function and instead generates a message.\n`auto` means the model can pick between generating a message or calling a function.\nSpecifying a particular function via `{"name": "my_function"}` forces the model to call that function.\n\n`none` is the default when no functions are present. `auto` is the default if functions are present.\n'
        ),
    ] = None
    functions: Annotated[
        Optional[List[ChatCompletionFunctions]],
        Field(
            description="Deprecated in favor of `tools`.\n\nA list of functions the model may generate JSON inputs for.\n",
            max_items=128,
            min_items=1,
        ),
    ] = None


class CreateThreadAndRunRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    assistant_id: Annotated[
        str,
        Field(
            description="The ID of the [assistant](/docs/api-reference/assistants) to use to execute this run."
        ),
    ]
    thread: Annotated[
        Optional[CreateThreadRequest],
        Field(description="If no thread is provided, an empty thread will be created."),
    ] = None
    model: Annotated[
        Optional[
            Union[
                str,
                Literal[
                    "gpt-4o",
                    "gpt-4o-2024-08-06",
                    "gpt-4o-2024-05-13",
                    "gpt-4o-2024-08-06",
                    "gpt-4o-mini",
                    "gpt-4o-mini-2024-07-18",
                    "gpt-4-turbo",
                    "gpt-4-turbo-2024-04-09",
                    "gpt-4-0125-preview",
                    "gpt-4-turbo-preview",
                    "gpt-4-1106-preview",
                    "gpt-4-vision-preview",
                    "gpt-4",
                    "gpt-4-0314",
                    "gpt-4-0613",
                    "gpt-4-32k",
                    "gpt-4-32k-0314",
                    "gpt-4-32k-0613",
                    "gpt-3.5-turbo",
                    "gpt-3.5-turbo-16k",
                    "gpt-3.5-turbo-0613",
                    "gpt-3.5-turbo-1106",
                    "gpt-3.5-turbo-0125",
                    "gpt-3.5-turbo-16k-0613",
                ],
            ]
        ],
        Field(
            description="The ID of the [Model](/docs/api-reference/models) to be used to execute this run. If a value is provided here, it will override the model associated with the assistant. If not, the model associated with the assistant will be used.",
            example="gpt-4o",
        ),
    ] = None
    instructions: Annotated[
        Optional[str],
        Field(
            description="Override the default system message of the assistant. This is useful for modifying the behavior on a per-run basis."
        ),
    ] = None
    tools: Annotated[
        Optional[List[Union[AssistantToolsCode, AssistantToolsFileSearch, AssistantToolsFunction]]],
        Field(
            description="Override the tools the assistant can use for this run. This is useful for modifying the behavior on a per-run basis.",
            max_items=20,
        ),
    ] = None
    tool_resources: Annotated[
        Optional[ToolResources3],
        Field(
            description="A set of resources that are used by the assistant's tools. The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file IDs, while the `file_search` tool requires a list of vector store IDs.\n"
        ),
    ] = None
    metadata: Annotated[
        Optional[Dict[str, Any]],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ] = None
    temperature: Annotated[
        Optional[float],
        Field(
            description="What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.\n",
            example=1,
            ge=0.0,
            le=2.0,
        ),
    ] = 1
    top_p: Annotated[
        Optional[float],
        Field(
            description="An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.\n\nWe generally recommend altering this or temperature but not both.\n",
            example=1,
            ge=0.0,
            le=1.0,
        ),
    ] = 1
    stream: Annotated[
        Optional[bool],
        Field(
            description="If `true`, returns a stream of events that happen during the Run as server-sent events, terminating when the Run enters a terminal state with a `data: [DONE]` message.\n"
        ),
    ] = None
    max_prompt_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of prompt tokens that may be used over the course of the run. The run will make a best effort to use only the number of prompt tokens specified, across multiple turns of the run. If the run exceeds the number of prompt tokens specified, the run will end with status `incomplete`. See `incomplete_details` for more info.\n",
            ge=256,
        ),
    ] = None
    max_completion_tokens: Annotated[
        Optional[int],
        Field(
            description="The maximum number of completion tokens that may be used over the course of the run. The run will make a best effort to use only the number of completion tokens specified, across multiple turns of the run. If the run exceeds the number of completion tokens specified, the run will end with status `incomplete`. See `incomplete_details` for more info.\n",
            ge=256,
        ),
    ] = None
    truncation_strategy: Optional[TruncationObject] = None
    tool_choice: Optional[AssistantsApiToolChoiceOption] = None
    parallel_tool_calls: Optional[ParallelToolCalls] = None
    response_format: Optional[AssistantsApiResponseFormatOption] = None


class RunStepObject(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The identifier of the run step, which can be referenced in API endpoints."
        ),
    ]
    object: Annotated[
        Literal["thread.run.step"],
        Field(description="The object type, which is always `thread.run.step`."),
    ]
    created_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run step was created."),
    ]
    assistant_id: Annotated[
        str,
        Field(
            description="The ID of the [assistant](/docs/api-reference/assistants) associated with the run step."
        ),
    ]
    thread_id: Annotated[
        str,
        Field(description="The ID of the [thread](/docs/api-reference/threads) that was run."),
    ]
    run_id: Annotated[
        str,
        Field(
            description="The ID of the [run](/docs/api-reference/runs) that this run step is a part of."
        ),
    ]
    type: Annotated[
        Literal["message_creation", "tool_calls"],
        Field(
            description="The type of run step, which can be either `message_creation` or `tool_calls`."
        ),
    ]
    status: Annotated[
        Literal["in_progress", "cancelled", "failed", "completed", "expired"],
        Field(
            description="The status of the run step, which can be either `in_progress`, `cancelled`, `failed`, `completed`, or `expired`."
        ),
    ]
    step_details: Annotated[
        Union[RunStepDetailsMessageCreationObject, RunStepDetailsToolCallsObject],
        Field(description="The details of the run step."),
    ]
    last_error: Annotated[
        LastError1,
        Field(
            description="The last error associated with this run step. Will be `null` if there are no errors."
        ),
    ]
    expired_at: Annotated[
        int,
        Field(
            description="The Unix timestamp (in seconds) for when the run step expired. A step is considered expired if the parent run is expired."
        ),
    ]
    cancelled_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run step was cancelled."),
    ]
    failed_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run step failed."),
    ]
    completed_at: Annotated[
        int,
        Field(description="The Unix timestamp (in seconds) for when the run step completed."),
    ]
    metadata: Annotated[
        Dict[str, Any],
        Field(
            description="Set of 16 key-value pairs that can be attached to an object. This can be useful for storing additional information about the object in a structured format. Keys can be a maximum of 64 characters long and values can be a maxium of 512 characters long.\n"
        ),
    ]
    usage: RunStepCompletionUsage


class Delta1(BaseModel):
    step_details: Annotated[
        Optional[
            Union[
                RunStepDeltaStepDetailsMessageCreationObject,
                RunStepDeltaStepDetailsToolCallsObject,
            ]
        ],
        Field(description="The details of the run step."),
    ] = None


class RunStepDeltaObject(BaseModel):
    id: Annotated[
        str,
        Field(
            description="The identifier of the run step, which can be referenced in API endpoints."
        ),
    ]
    object: Annotated[
        Literal["thread.run.step.delta"],
        Field(description="The object type, which is always `thread.run.step.delta`."),
    ]
    delta: Annotated[
        Delta1,
        Field(description="The delta containing the fields that have changed on the run step."),
    ]


class ListRunStepsResponse(BaseModel):
    object: Annotated[str, Field(example="list")]
    data: List[RunStepObject]
    first_id: Annotated[str, Field(example="step_abc123")]
    last_id: Annotated[str, Field(example="step_abc456")]
    has_more: Annotated[bool, Field(example=False)]


class RunStepStreamEvent1(BaseModel):
    event: Literal["thread.run.step.created"]
    data: RunStepObject


class RunStepStreamEvent2(BaseModel):
    event: Literal["thread.run.step.in_progress"]
    data: RunStepObject


class RunStepStreamEvent3(BaseModel):
    event: Literal["thread.run.step.delta"]
    data: RunStepDeltaObject


class RunStepStreamEvent4(BaseModel):
    event: Literal["thread.run.step.completed"]
    data: RunStepObject


class RunStepStreamEvent5(BaseModel):
    event: Literal["thread.run.step.failed"]
    data: RunStepObject


class RunStepStreamEvent6(BaseModel):
    event: Literal["thread.run.step.cancelled"]
    data: RunStepObject


class RunStepStreamEvent7(BaseModel):
    event: Literal["thread.run.step.expired"]
    data: RunStepObject


class RunStepStreamEvent(BaseModel):
    __root__: Union[
        RunStepStreamEvent1,
        RunStepStreamEvent2,
        RunStepStreamEvent3,
        RunStepStreamEvent4,
        RunStepStreamEvent5,
        RunStepStreamEvent6,
        RunStepStreamEvent7,
    ]


class AssistantStreamEvent(BaseModel):
    __root__: Annotated[
        Union[
            ThreadStreamEvent,
            RunStreamEvent,
            RunStepStreamEvent,
            MessageStreamEvent,
            ErrorEvent,
            DoneEvent,
        ],
        Field(
            description='Represents an event emitted when streaming a Run.\n\nEach event in a server-sent events stream has an `event` and `data` property:\n\n```\nevent: thread.created\ndata: {"id": "thread_123", "object": "thread", ...}\n```\n\nWe emit events whenever a new object is created, transitions to a new state, or is being\nstreamed in parts (deltas). For example, we emit `thread.run.created` when a new run\nis created, `thread.run.completed` when a run completes, and so on. When an Assistant chooses\nto create a message during a run, we emit a `thread.message.created event`, a\n`thread.message.in_progress` event, many `thread.message.delta` events, and finally a\n`thread.message.completed` event.\n\nWe may add additional events over time, so we recommend handling unknown events gracefully\nin your code. See the [Assistants API quickstart](/docs/assistants/overview) to learn how to\nintegrate the Assistants API with streaming.\n'
        ),
    ]
